package application2.aspects;

import application2.interfaces.*;
import library.interfaces.*;

import java.util.*;

import library.util.Debugger;

public aspect FlatModelWithOptimisticConcurrencyContextAspect 
{
	declare parents: FlatModelWithOptimisticConcurrencyContext	implements /*Tracing,*/ 
																			Deferring, 
																			/*OutcomeAware,*/ 
																			Recovering, 
																			RecoveringDeferred,
																			Pausable;
																			
	// Start with 20
	public static int timeStampGenerator = 20;
	
	public int FlatModelWithOptimisticConcurrencyContext.timeStamp = 0;

	   
	public synchronized boolean FlatModelWithOptimisticConcurrencyContext.canBeValidated()
	{
		boolean canBeValidated = true;
		
		Set<Object> writeSet = this.getModifiedObjects();
		Vector<FlatModelWithOptimisticConcurrencyContext> conflictingContexts;
		
		for(Object object: writeSet)
		{
			FlatModelWithOptimisticConcurrencyObject transactionalObject = (FlatModelWithOptimisticConcurrencyObject) object;
			
			conflictingContexts = getContextsThatReadObject(transactionalObject);
			
			for(FlatModelWithOptimisticConcurrencyContext context: conflictingContexts)
			{
				if(context.timeStamp <= timeStampGenerator && context.timeStamp > this.timeStamp)
				{
					canBeValidated = false;
					break;
				}
			}
			
			if(!canBeValidated)
				break;
		}
		
		return canBeValidated;
	}
	
	private Vector<FlatModelWithOptimisticConcurrencyContext> 
				FlatModelWithOptimisticConcurrencyContext.getContextsThatReadObject(FlatModelWithOptimisticConcurrencyObject transactionalObject)
	{
		Set<Context> allContexts = transactionalObject.getAllContexts();
		Vector<FlatModelWithOptimisticConcurrencyContext> conflictingContexts = new Vector<FlatModelWithOptimisticConcurrencyContext>();
		
		for(Context currentContext: allContexts)
		{
			FlatModelWithOptimisticConcurrencyContext context = 
					(FlatModelWithOptimisticConcurrencyContext) currentContext;
			
			if(context.hasReadObject(transactionalObject.getMyName()))
			{
				conflictingContexts.add(context);
			}
		}
		
		return conflictingContexts;
	}
	
	pointcut contextCreation(FlatModelWithOptimisticConcurrencyContext context):
		target(context) && execution(public FlatModelWithOptimisticConcurrencyContext+.new(..)) ;

	after(FlatModelWithOptimisticConcurrencyContext context) : contextCreation(context)
	{
		Debugger.printMessage("OptimisticValidation",Debugger.CONTEXT_ASPECT,context,"Constructor Execution (after) call has been intercepted.",2);
		
		context.timeStamp = timeStampGenerator;
		
		Debugger.printMessage("OptimisticValidation",Debugger.CONTEXT_ASPECT,context,"The context gets the timestamp: " + timeStampGenerator + ".",1);
		
		Debugger.printMessage("OptimisticValidation",Debugger.CONTEXT_ASPECT,context,"Constructor Execution (after) call has been released.",2);
	}
	
	pointcut totalAgreement(FlatModelWithOptimisticConcurrencyContext context):
		target(context) && call(public * Context.inactivateContext(..));
	
	before(FlatModelWithOptimisticConcurrencyContext context) : totalAgreement(context)
	{
		Debugger.printMessage("OptimisticValidation",Debugger.CONTEXT_ASPECT,context,"The inactivation call (before) has been intercepted",2);
		
		Debugger.printMessage("OptimisticValidation",Debugger.CONTEXT_ASPECT,context,"The validation phase is about to be performed.",1);
		
		if(context.canBeValidated())
		{
			timeStampGenerator++;
			context.timeStamp = timeStampGenerator;
			
			Debugger.printMessage("OptimisticValidation",Debugger.CONTEXT_ASPECT,context,"The context is validated, and gets the timestamp: " + timeStampGenerator + ".",1);
		}
		else
		{
			Debugger.printMessage("OptimisticValidation",Debugger.CONTEXT_ASPECT,context,"The context CANNOT be validated.",1);
			
			context.setOutcome(false);
		}
		
		Debugger.printMessage("OptimisticValidation",Debugger.CONTEXT_ASPECT,context,"The inactivation call (before) has been released",2);
	}
}
