package glassbox.agent.control;

import glassbox.analysis.api.OperationSummary;
import glassbox.track.OperationTracker;

import java.util.HashSet;
import java.util.Set;

import org.jmock.Mock;
import org.jmock.MockObjectTestCase;

public class ServiceUnitTest extends MockObjectTestCase {
    GlassboxServiceImpl service;
    Mock mockTracker;
    Set s;
    Mock operation1; 
    Mock operation2;
    
    public void setUp() {
        service = new GlassboxServiceImpl();
        mockTracker = mock(OperationTracker.class);
        service.setOperationTracker((OperationTracker)mockTracker.proxy());
        s = new HashSet();
        operation1 = mock(OperationSummary.class, "operation1"); 
        operation2 = mock(OperationSummary.class, "operation2");
        s.add(operation1.proxy());
        s.add(operation2.proxy());
    }
    
    public void testProblemCount() {
        mockTracker.expects(once()).method("getProblemOperations").will(returnValue(s));
        assertEquals(s.size(), service.countProblemOperations());
    }
    
    public void testAccumulatedTime() {
        mockTracker.stubs().method("getOperations").will(returnValue(s));
        operation1.stubs().method("getAvgExecutionTime").will(returnValue(1.2));
        operation1.stubs().method("getCount").will(returnValue(3));
        operation2.stubs().method("getAvgExecutionTime").will(returnValue(0.2));
        operation2.stubs().method("getCount").will(returnValue(2));
        assertEquals(4000L, service.timeSpentOverallInMillis());
    }
    
    public void testErrorProblem() {
        RuntimeException failure = new RuntimeException("failure");
        mockTracker.expects(once()).method("getProblemOperations").will(throwException(failure));
        try {
            service.countProblemOperations();
            fail("swallowed exception");
        } catch (RuntimeException rte) {
            assertSame(failure, rte);
        }
    }

    public void testErrorTimeSpent() {
        RuntimeException failure = new RuntimeException("failure");
        mockTracker.expects(once()).method("getOperations").will(throwException(failure));
        try {
            service.timeSpentOverallInMillis();
            fail("swallowed exception");
        } catch (RuntimeException rte) {
            assertSame(failure, rte);
        }
    }
}
