package glassbox.installer.perContainer;

import glassbox.installer.BaseGlassboxInstaller;
import glassbox.installer.GlassboxInstallerFactory;

import java.io.File;
import java.io.FilenameFilter;
import java.net.MalformedURLException;
import java.net.URL;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * Important? system properties:
 * weblogic.Name=examplesServer
 * 
 * This one property appears to be it...
 * 
 */
public class Weblogic8GlassboxInstaller extends BaseGlassboxInstaller {
    private static final Log log = LogFactory.getLog(Weblogic8GlassboxInstaller.class);

	/**
	 * Assume Weblogic 8 is always started in the right directory...
	 */
	public File getContainerHome() {
        return new File(".").getAbsoluteFile();
	}
	
	public File getDefaultBinDirectory() {
		return getContainerHome();
	}

	public String getTargetSystemLabel() {
		return "WebLogic";
	}
	
	public String getDefaultScriptPrefix() {
		String domainName = System.getProperty("weblogic.Name");
		return "start"+domainName.substring(0, 1).toUpperCase()+domainName.substring(1, domainName.length());
	}	

	protected String getJavaOptsName() {
		return "JAVA_OPTIONS";
	}
	
    public boolean matchesContext(ServletContext context) {	    
		boolean webLogicName = System.getProperty("weblogic.Name")!=null;
		boolean startsWith = context.getServerInfo().startsWith("WebLogic Server 8");
		boolean classloaderMatch = checkClassloadersNameMatchPrefix("weblogic.");
		boolean result = 
			webLogicName &&
			startsWith && 
			classloaderMatch;
		
		log.info("Detect Weblogic8: webLogicName="+webLogicName+", startsWith="+startsWith+", classloaderMatch="+classloaderMatch);

		return result;
	}
    
	public String getPlatformExtension() {
		if (isWindows())
			return ".cmd";
		else
			return ".sh";
	}

	public String getInstallerResultsPageName() {
		return "installerResults_Weblogic8Installer";
	}
	
}
