package glassbox.installer.perContainer;

import glassbox.installer.BaseGlassboxInstaller;

import java.io.File;
import java.io.FilenameFilter;
import java.net.MalformedURLException;
import java.net.URL;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * Important? system properties:
 * wls.home=/home/jheintz/projects/glassbox/glassbox-smoke/bea/wlserver_10.0/server
 * platform.home=/home/jheintz/projects/glassbox/glassbox-smoke/bea/wlserver_10.0
 * weblogic.home=/home/jheintz/projects/glassbox/glassbox-smoke/bea/wlserver_10.0/server
 * weblogic.Name=examplesServer
 * 
 * ServletContext.getResource("/")="file:/home/jheintz/projects/glassbox/glassbox-smoke/bea/wlserver_10.0/samples/domains/wl_server/servers/examplesServer/tmp/_WL_user/_appsdir_glassbox_war/346ozk/war/"
 * 
 */
public class WeblogicGlassboxInstaller extends BaseGlassboxInstaller {

	public File getContainerHome() {
        return getWebLogicDomainDir(this.getContext());
	}
	
	public String getTargetSystemLabel() {
		return "WebLogic";
	}
	
	public String getDefaultScriptPrefix() {
		return "startWebLogic";
	}	

	protected String getJavaOptsName() {
		return "JAVA_OPTIONS";
	}
	
    private static final Log log = LogFactory.getLog(WeblogicGlassboxInstaller.class);
	
    public boolean matchesContext(ServletContext context) {
    	
		boolean startsWith = context.getServerInfo().startsWith("WebLogic Server");
		boolean domainDir = getWebLogicDomainDir(context) != null;
		boolean classloader = checkClassloadersNameMatchPrefix("weblogic.");
		
		boolean result = (startsWith && 
							domainDir) && 
							classloader;
		
		log.info("Detect Weblogic: startsWith="+startsWith+", domainDir="+domainDir+", classloader="+classloader);
		
		return result;
	}

	protected File getInstallerResourceDir() {
		File root = getExplodedResourceFile(this.getContext());
		return new File(root, "install");
	}
	
	/**
	 * Search for the domains directory in a Weblogic installation. Returns null if not found.
	 */
	public static File getWebLogicDomainDir(ServletContext context) {
	    File startDir = getExplodedResourceFile(context);
	    log.info("Exploded WebLogic resource directory is "+startDir.getAbsolutePath());
	    
	    try {
	    	// return 7 dirs up...
	    	return startDir.getParentFile().getParentFile().getParentFile().getParentFile().getParentFile().getParentFile().getParentFile();
	    } catch (NullPointerException ex) {
	    	return null;
	    }
	}
	
	public static File getExplodedResourceFile(ServletContext context) {
		try {
			URL url = context.getResource("/");
			String path = url.getFile();
			
			if (path.startsWith("file:"))
				path = path.substring("file:".length(), path.length());

			File file = new File(path);
			
			if (!file.exists())
				log.error("File doesn't exist: "+file);
			
			return file;
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}
}
