/*
 * Copyright (c) 2005-2006 Glassbox Corporation, Contributors. All rights reserved.
 * This program along with all accompanying source code and applicable materials are made available under the 
 * terms of the Lesser Gnu Public License v2.1, which accompanies this distribution and is available at 
 * http://www.gnu.org/licenses/lgpl.txt
 */
package glassbox.monitor.ui;

import glassbox.response.Response;

import java.io.Serializable;
import java.lang.reflect.Method;

// this version supports both DWR 1.x and 2.x 
public aspect DwrMonitor extends MvcFrameworkMonitor {
    private static final String REQUEST_TYPE = "AjaxRequest"; 
    /** Make DWR operations of higher priority: this lets them win over, e.g., Spring wrapper controllers */
    private static final Integer DWR_PRIORITY = new Integer(Response.DEFAULT_PRIORITY.intValue() + 5);
    
    // DWR 1.x
    private pointcut inExecQuery() : 
        (within(uk.ltd.getahead.dwr.impl.ExecuteQuery) || within(uk.ltd.getahead.dwr.ExecuteQuery)) &&
        withincode(* execute(..));
    
    //DWR 2.x
    private pointcut inAjaxFilter() : 
        within(org.directwebremoting.impl.ExecuteAjaxFilter) &&
        withincode(* doFilter(..));
    
    public pointcut dwrCall(Method method, Object receiver, Object[] params) : 
        (inExecQuery() || inAjaxFilter()) && 
        call(* Method.invoke(..)) && args(receiver, params) && target(method);

    protected pointcut monitorEnd() : dwrCall(*, *, *);
    
    public DwrMonitor() {
        super(REQUEST_TYPE);
    }
    
    // this could be reused as "reflection method"
    before(Method method, Object receiver, Object[] params) : dwrCall(method, receiver, params) {
        Serializable key = getMethodDescriptor(method.getDeclaringClass().getName(), method.getName());
        Response response = createResponse(key, getLayer());
        response.set(Response.PARAMETERS, params);
        response.set(Response.OPERATION_PRIORITY, DWR_PRIORITY);
        response.start();
    }    

    // we could also add dwr filter tracking, like for servlet filters 
}
