/**
 *
 */
package helpers;
/* <postprocessor class="oaw.xpand2.output.JavaBeautifier"/>*/



import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.UniqueEList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.Gate;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageEnd;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageMerge;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterDirectionKind;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.SendOperationEvent;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.internal.impl.CallEventImpl;

/**
 * @author Thorsten Kamann <thorsten.kamann@googlemail.com> Helper class for the
 *         helper extension of the JavaBasic-Generator.
 * @since 1.0.0
 */
public class Helper {
	private static ThemeMerger themeMerger = new ThemeMerger();
	/**
	 * Calculate the full qualified packagename of the given type
	 * @param type The <code>org.eclipse.uml2.uml.Type</code> to calculate the
	 * full qualified name for
	 * @return The full qualified name as <code>java.lang.String</code>
	 */
	public static String getFQNPackageName(Type type) {
		String pn = "";
		Package p = type.getPackage();

		while (p != null) {
			pn = p.getName() + "." + pn;
			try {
				p = (Package) p.getOwner();
				if (p instanceof Model) {
					p = null;
				}
			} catch (Exception ex) {
				p = null;
			}
		}

		if (pn.endsWith(".")) {
			pn = pn.substring(0, pn.length() - 1);
		}
		return pn;
	}

	/**
	 * Calculate the full qualified packagepath of the given type
	 * @param type The <code>org.eclipse.uml2.uml.Type</code> to calculate the
	 * full qualified path for
	 * @return The full qualified path as <code>java.lang.String</code>
	 */
	public static String getFQNPackagePath(Type type) {
		String path = getFQNPackageName(type);

		path = path.replaceAll("\\.", "/");
		return path;
	}
	
	

	/**
	 * Collects all attributes the classifier and their interfaces have.
	 * @param classifier The classifier the attributes should be collected
	 * @return An <code>EList</code> with all collected attributes
	 */
	public static EList getAllAttributes(Classifier classifier){
		EList attributes = new UniqueEList();

		attributes.addAll(classifier.getAttributes());
		if (classifier instanceof org.eclipse.uml2.uml.Class){
			Class clazz = (Class) classifier;
			for (int i=0;i<clazz.getImplementedInterfaces().size();i++){
				attributes.addAll(getAllInterfaceAttributes((Interface) clazz.getImplementedInterfaces().get(i)));
			}
		}
		return attributes;
	}

	/**
	 * Collects all operations the classifier and their interfaces have.
	 * @param classifier The classifier the operations should be collected
	 * @return An <code>EList</code> with all collected operations
	 */
	public static EList getAllOperations(Classifier classifier){
		EList operations = new UniqueEList();

		operations.addAll(classifier.getOperations());
		if (classifier instanceof org.eclipse.uml2.uml.Class){
			Class clazz = (Class) classifier;
			for (int i=0;i<clazz.getImplementedInterfaces().size();i++){
				operations.addAll(getAllInterfaceOperations((Interface) clazz.getImplementedInterfaces().get(i)));
			}
		}
		return operations;
	}

	/**
	 * Collects all dependencies the classifier and their interfaces have.
	 * @param classifier The classifier the dependencies should be collected
	 * @return An <code>EList</code> with all collected dependencies
	 */
	public static EList getAllDependencies(Classifier classifier){
		EList dependencies = new UniqueEList();

		dependencies.addAll(classifier.getClientDependencies());
		if (classifier instanceof org.eclipse.uml2.uml.Class){
			Class clazz = (Class) classifier;
			for (int i=0;i<clazz.getImplementedInterfaces().size();i++){
				dependencies.addAll(getAllInterfaceAssociations((Interface) clazz.getImplementedInterfaces().get(i)));
			}
		}

		EList depsFiltered = new UniqueEList();
		for (Iterator iter = dependencies.iterator(); iter.hasNext();) {
			Object element = (Object) iter.next();
			if (!(element instanceof InterfaceRealization)){
				depsFiltered.add(element);
			}
		}
		return depsFiltered;
	}

	/**
	 * Collects all associations the classifier and their interfaces have.
	 * @param classifier The classifier the associations should be collected
	 * @return An <code>EList</code> with all collected associations
	 */
	public static EList getAllAssociations(Classifier classifier){
		EList associations = new UniqueEList();

		associations.addAll(classifier.getAssociations());
		if (classifier instanceof org.eclipse.uml2.uml.Class){
			Class clazz = (Class) classifier;
			for (int i=0;i<clazz.getImplementedInterfaces().size();i++){
				associations.addAll(getAllInterfaceAssociations((Interface) clazz.getImplementedInterfaces().get(i)));
			}
		}
		return associations;
	}

	/**
	 * Collects all attributes of the given interface needed to implement (creating fields
	 * and getter/setter methods) by a implementing class.
	 * @param iFace The interface to collect the operatiosn from
	 * @return An <code>EList</code> with all collected attributes
	 */
	private static EList getAllInterfaceAttributes(Interface iFace){
		EList attributes = new UniqueEList();

		attributes.addAll(iFace.getAllAttributes());
		if (iFace.getGeneralizations().size() > 0){
			for (int i=0;i<iFace.getGeneralizations().size();i++){
				attributes.addAll(
						getAllInterfaceAttributes(
								(Interface) ((Generalization) iFace.getGeneralizations().get(i)).getGeneral()));
			}
		}
		return attributes;
	}

	/**
	 * Collects all operation of the given interface needed to implement by a
	 * implementing class.
	 * @param iFace The interface to collect the operations from
	 * @return An <code>EList</code> with all collected operations
	 */
	private static EList getAllInterfaceOperations(Interface iFace){
		EList operations = new UniqueEList();

		operations.addAll(iFace.getAllOperations());
		if (iFace.getGeneralizations().size() > 0){
			for (int i=0;i<iFace.getGeneralizations().size();i++){
				operations.addAll(
						getAllInterfaceOperations(
								(Interface) ((Generalization) iFace.getGeneralizations().get(i)).getGeneral()));
			}
		}
		return operations;
	}

	/**
	 * Collects all associations of the given interface needed to implement by a
	 * implementing class.
	 * @param iFace The interface to collect the associations from
	 * @return An <code>EList</code> with all collected associations
	 */
	private static EList getAllInterfaceAssociations(Interface iFace){
		EList associations = new UniqueEList();

		associations.addAll(iFace.getAssociations());
		if (iFace.getGeneralizations().size() > 0){
			for (int i=0;i<iFace.getGeneralizations().size();i++){
				associations.addAll(
						getAllInterfaceAssociations(
								(Interface) ((Generalization) iFace.getGeneralizations().get(i)).getGeneral()));
			}
		}
		return associations;
	}

	/**
	 * Collects all dependencies of the given interface needed to implement by a
	 * implementing class.
	 * @param iFace The interface to collect the dependencies from
	 * @return An <code>EList</code> with all collected dependencies
	 */
	private static EList getAllInterfaceDependencies(Interface iFace){
		EList dependencies = new UniqueEList();

		dependencies.addAll(iFace.getClientDependencies());
		if (iFace.getGeneralizations().size() > 0){
			for (int i=0;i<iFace.getGeneralizations().size();i++){
				dependencies.addAll(
						getAllInterfaceAssociations(
								(Interface) ((Generalization) iFace.getGeneralizations().get(i)).getGeneral()));
			}
		}
		return dependencies;
	}

	public static String formatComment(Element element,
			String prefix) {
		String comment = "";

		comment = getFormatedComment(element.getOwnedComments(), prefix, null);
		return comment;
	}

	public static String formatParameterComment(Parameter parameter,
			String prefix, String annotation) {
		String comment = "";

		if (parameter.getOwnedComments().isEmpty()) {
			comment = prefix + "@" + annotation + " " + parameter.getName();
		} else {
			comment = getFormatedComment(parameter.getOwnedComments(), prefix,
					annotation+" "+parameter.getName());
		}
		return comment;
	}

	public static String getDefaultValue(Property property){
		ValueSpecification vs = property.getDefaultValue();

		if (vs == null){
			if (property.getType() instanceof PrimitiveType){
				PrimitiveType pt = (PrimitiveType) property.getType();
				if (pt.getName().equals("int")){
					return "0";
				}
			}
			return "null";
		}else{
			if (vs.getType() == null){
				return vs.stringValue();
			}else if (vs.getType().getName().equals("String")){
				return vs.stringValue();
			}else if (vs.getType().getName().equals("Integer")){
				return vs.integerValue()+"";
			}else if (vs.getType().getName().equals("Boolean")){
				return vs.booleanValue()+"";
			}
			return vs.stringValue();
		}
	}

	private static String getFormatedComment(EList comments, String prefix,
			String annotation) {
		String s = "";

		if (!comments.isEmpty()) {
			for (int i = 0; i < comments.size(); i++) {
				if (!"".equals(s)){
					s += "\n";
				}
				Comment c = (Comment) comments.get(i);
				String cs = c.getBody();
				cs = cs.replaceAll("\\n", "\n" + prefix);
				s += prefix;
				if (annotation != null && !"".equals(annotation)) {
					s += "@" + annotation;
				}
				s += " " + cs;
			}
		}else{
			s = "* ";
		}
		return s;
	}
	
	
	public static EList getAllMerges(org.eclipse.uml2.uml.Model model)
	{
		EList<Dependency> merges = new UniqueEList();
		
		/******/
		org.eclipse.uml2.uml.Profile themeUMLProfile = model.getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoMerge = themeUMLProfile.getOwnedStereotype("merge"); 

		// Find all merges
		for (Element element : model.allOwnedElements())
		{
			if (element.isStereotypeApplied(stereoMerge)
					&& element instanceof Dependency)
				merges.add((Dependency) element);
		}
		
		for (Dependency dependency : merges)
		{
			
			
			int sources = dependency.getSources().size();
			
			if(dependency.getSources().get(0) != null)
			{
				if (dependency.getSources().get(0) instanceof Dependency)
				{		
					Dependency source = (Dependency) dependency.getSources().get(0);
					
				}	
				else if (dependency.getSources().get(0) instanceof Package)
				{
					Package source = (Package) dependency.getSources().get(0);
					
				}
			}
			int targets = dependency.getTargets().size();
			Package target = (Package) dependency.getTargets().get(0);
			
		}
		
		return merges;
	}
	
	public static EList getAllMergeDependencies(org.eclipse.uml2.uml.Dependency merge)
	{
		EList depenencies = new UniqueEList();
		/*****/
		return depenencies;
	}
	
	public static EList getAllAspectThemes(org.eclipse.uml2.uml.Model model)
	{
		EList themes = new UniqueEList();

		
		
		org.eclipse.uml2.uml.Profile themeUMLProfile = model.getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereotheme = themeUMLProfile.getOwnedStereotype("theme"); 
		org.eclipse.uml2.uml.Stereotype stereobind = themeUMLProfile.getOwnedStereotype("bind"); 

		// Find all elements in the theme
		for (Element element : model.allOwnedElements())
		{
			if (element.isStereotypeApplied(stereotheme)
					&& element instanceof Package)
			{
				Package theme = (Package) element;
				
				
				EList<Dependency> bindings = new UniqueEList();
				for (Element ele : theme.allOwnedElements())
				{
					
					//
					if (ele.isStereotypeApplied(stereobind)
							&& ele instanceof Dependency)
					{
						bindings.add((Dependency)ele);
						
					}
				}
				if(bindings.size() > 0)
				{
					themes.add(theme);
				}
			}	
		}
		
		return themes;
	}	
	
	public static EList getAllThemes(org.eclipse.uml2.uml.Dependency mainDependency)
	{
		EList themes = new UniqueEList();

		
		
		/*********/
		
		
		Model model = mainDependency.getModel();
		
		org.eclipse.uml2.uml.Profile themeUMLProfile = model.getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoMerge = themeUMLProfile.getOwnedStereotype("merge"); 

		LinkedList<Dependency> merges = new LinkedList<Dependency>();

		// Find all merges
		for (Element element : model.allOwnedElements())
		{
			if (element.isStereotypeApplied(stereoMerge)
					&& element instanceof Dependency)
				merges.add((Dependency) element);
		}
		
		Dependency mainMergeDependency = null;

		// Find main merge dependency (the one with the stereotypes applied)
		for (Dependency dep : merges)
		{
			if ((dep.getSources().get(0) instanceof Package)
					&& (dep.getTargets().get(0) instanceof Package))
				mainMergeDependency = dep;
		}

		themes.add((Package) mainMergeDependency.getSources().get(0));
		themes.add((Package) mainMergeDependency.getTargets().get(0));

		// Now find all themes involved in merge
		// Find any dependencies that are merges attached to this dependency
		// (This emulates the n-ary dependency)

		for (Dependency dep : merges)
		{
			if (dep.getSources().get(0) == mainMergeDependency)
				themes.add((Package) dep.getTargets().get(0));
		}

		/*********/
		
		return themes;
	}
	
	public static EList getAllThemes(org.eclipse.uml2.uml.Model model)
	{
		EList<Package> themes = new UniqueEList();

		/*********/
		
		
		//Model model = mainDependency.getModel();
		
		org.eclipse.uml2.uml.Profile themeUMLProfile = model.getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 

		LinkedList<Dependency> merges = new LinkedList<Dependency>();

		// Find all merges
		for (Element element : model.allOwnedElements())
		{
			if (element.isStereotypeApplied(stereoTheme)
					&& element instanceof Package)
				themes.add((Package) element);
		}

		/*********/
		
		return themes;
	}
	
	public static EList getAllAspectBindings(org.eclipse.uml2.uml.Package theme)
	{
		EList<String> bindings = new UniqueEList();
		org.eclipse.uml2.uml.Profile themeUMLProfile = theme.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 
		org.eclipse.uml2.uml.Stereotype stereoBinding = themeUMLProfile.getOwnedStereotype("bind");
		if(theme.isStereotypeApplied(stereoTheme))
		{
			for (Element element : theme.allOwnedElements())
			{
				if (element.isStereotypeApplied(stereoBinding)
						&& element instanceof Dependency)
				{
					Dependency dep = (Dependency)element;
					bindings.add(dep.getName());
				}
			}
		}
		return bindings; 
	}
	
	public static EList getAllAspectBindings(org.eclipse.uml2.uml.Class cl)
	{
		org.eclipse.uml2.uml.Package theme = cl.getPackage();
		return getAllAspectBindings(theme);
	}
	
	public static String getReturnTypeOnBinding(org.eclipse.uml2.uml.Package theme)
	{
		String retType = "void ";
		String bindings = "";
		org.eclipse.uml2.uml.Profile themeUMLProfile = theme.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 
		org.eclipse.uml2.uml.Stereotype stereoBinding = themeUMLProfile.getOwnedStereotype("bind");
		boolean first = true;
		if(theme.isStereotypeApplied(stereoTheme))
		{
			
			
			for (Element element : theme.allOwnedElements())
			{
				if (element instanceof Dependency)
				{
					
					Dependency dep = (Dependency)element;
					if (dep.isStereotypeApplied(stereoBinding))
					{
						if (dep.getSources().get(0) == theme)
						{
							String b = (String) dep.getValue(stereoBinding, "binding");
							b = b.substring(b.indexOf((int)'<') + 1,b.indexOf((int)'>'));
							
							for (Element e : theme.getModel().allOwnedElements())
							{
								if (e instanceof Class)
								{
									
									if (((Class)e).getName().equals(b.substring(0, b.indexOf((int)'.'))))
									{
										for (Operation o : ((Class)e).getOperations())
										{
											
											if (o.getName().equals( b.substring(b.indexOf((int)'.') + 1, b.length() - 2) ))
											{
												if (o.getReturnResult().getType() != null)
												{
													retType = o.getReturnResult().getType().getName() + " ";
												}
											}
										}
									}
								}
							}
							b = getBaseThemeName(theme.getModel()) + "." + b;
							b = retType + b;
							if (!first){
								bindings = bindings + " || ";
							}
							first = false;
							
							bindings = bindings +"call("+b+")";
						}
					}
				}
			}
		}
		return retType; 
	}
	public static String getPointcuts(org.eclipse.uml2.uml.Package theme)
	{
		String bindings = "";
		org.eclipse.uml2.uml.Profile themeUMLProfile = theme.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 
		org.eclipse.uml2.uml.Stereotype stereoBinding = themeUMLProfile.getOwnedStereotype("bind");
		boolean first = true;
		try
		{
		if(theme.isStereotypeApplied(stereoTheme))
		{
			for (Element element : theme.allOwnedElements())
			{
				if (element instanceof Dependency)
				{
					Dependency dep = (Dependency)element;
					if (dep.isStereotypeApplied(stereoBinding))
					{
						if (dep.getSources().get(0) == theme)
						{
							String b = (String) dep.getValue(stereoBinding, "binding");
							ArrayList<String> ops = new ArrayList<String>();
							ArrayList<String> returnTypes = new ArrayList<String>();
							//this is the type
							String type = b.substring(b.indexOf("<")+1, b.indexOf((int)'.'));
							//this is the rest of it
							String temp = b.substring(b.indexOf((int)'.') + 1, b.length() - 2);
							//if more than 1 operations is binded
							//e.g. {generatePulse(),sensePulse()}
							//note - return type of both ops could be different
							if(temp.indexOf("{")!=-1)
							{
								//get rid of {}
								temp = temp.substring(1, temp.length()-1);
								String[] res = temp.split("\\),");
								//op1(), op2(), op3()
								//becomes op1( op2( op3()
								for(int k = 0; k<res.length; k++)
								{
									if(k!=res.length-1)
										ops.add(res[k]+ ")");
									else
										ops.add(res[k]);
								}
							}
							else
								ops.add(temp);
							for(String operation : ops)
							{
								String retType = "void ";
								for (Element e : theme.getModel().allOwnedElements())
								{
									if (e instanceof Class)
									{
										if (((Class)e).getName().equals(type))
										{
											for (Operation o : ((Class)e).getOperations())
											{
												if (o.getName().equals(operation.substring(0, operation.indexOf("("))))
												{
													if (o.getReturnResult()!=null && o.getReturnResult().getType() != null)
													{
														retType = o.getReturnResult().getType().getName();
													}
												}
											}
										}
									}
								}
								returnTypes.add(retType);
							}
							String base = getBaseThemeName(theme.getModel()).toLowerCase();
							for(int m = 0; m<ops.size(); m++)
							{
								if (!first){
									bindings = bindings + " || \n\t\t\t\t\t\t\t\t";
								}
								first = false;
								bindings = bindings +"call("+ returnTypes.get(m) + " " + base + "." + type + "." + ops.get(m) + ")";
							}
							String templateOp = getTemplateOp(theme);
							//this is a pretty bad practice but the whole Helper is a mess
							//couldn't find any better workaround
							//this piece of code adds the binding of input parameters
							String params = templateOp.substring(templateOp.indexOf("(")+1, templateOp.indexOf(")"));
							//params is something like String a, int b, double c
							if(params.length()>0)
							{
								//split params by the , sign
								String[] list = params.split(", ");
								for(int k = 0; k<list.length; k++)
								{
									//split to obtain the type and the name
									String[] typeName = list[k].split(" ");
									bindings += "&&args(" + typeName[1] + ")";
								}
							}
						}
					}
				}
			}
		}
		return bindings;
		}
		catch (NullPointerException npe)
		{
			npe.printStackTrace();
			return "";
		}
	}
	
	public static boolean isAdvice(org.eclipse.uml2.uml.Operation op)
	{
		boolean result = false;
		org.eclipse.uml2.uml.Profile themeUMLProfile = op.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoOp = themeUMLProfile.getOwnedStereotype("advice"); 
		result = op.isStereotypeApplied(stereoOp);
		return result;
	}
	public static boolean isAspect(org.eclipse.uml2.uml.Package theme)
	{
		boolean result = false;
		org.eclipse.uml2.uml.Profile themeUMLProfile = theme.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 
		org.eclipse.uml2.uml.Stereotype stereoBinding = themeUMLProfile.getOwnedStereotype("bind"); 
		
		
		if(theme.isStereotypeApplied(stereoTheme))
		{
			
			for (Element element : theme.allOwnedElements())
			{
				if (element.isStereotypeApplied(stereoBinding)
						&& element instanceof Dependency)
				{
					result = true;
					break;
				}
			}
		}
		return result;
	}
	
	public static boolean isAspect(org.eclipse.uml2.uml.Class cl)
	{
		boolean result = false;
		org.eclipse.uml2.uml.Profile themeUMLProfile = cl.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoAdvice = themeUMLProfile.getOwnedStereotype("advice");  
		
			for (Element element : cl.allOwnedElements())
			{
			
				if (element.isStereotypeApplied(stereoAdvice))
				{
					result = true;
					break;
				}
				//
			}
		
		return result;
	}
	
	public static String getTemplate(org.eclipse.uml2.uml.Package theme){
		org.eclipse.uml2.uml.Profile themeUMLProfile = theme.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 
		String t = theme.getValue(stereoTheme, "template").toString();
		t = t.trim();
		//get rid of < and > in the template
		t = t.substring(1, t.length() - 1);
		return t;
	}
	public static String getTemplateClass(org.eclipse.uml2.uml.Package theme){
		String result = getTemplate(theme);
		return result.substring(0, result.indexOf((int)'.'));
	}
	public static String getTemplateOp(org.eclipse.uml2.uml.Package theme)
	{
		String result = getTemplate(theme);
		result = result.substring(result.indexOf((int)'.') + 1);
		return result;
	}
	public static String getType(org.eclipse.uml2.uml.Operation advice){
		
		return "before";
		/*org.eclipse.uml2.uml.Profile themeUMLProfile = advice.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("advice");
		String result = advice.getValue(stereoTheme, "Type").toString().toLowerCase();
		if (!((result.equals("before"))
				||(result.equals("after"))
				||(result.equals("around")))){
			
			result = "before";
		}
		return result;*/
	}
	public static String getStuffNOW(org.eclipse.uml2.uml.Operation op)
	{
		org.eclipse.uml2.uml.Profile themeUMLProfile = op.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("advice");
		String result = op.getValue(stereoTheme, "type").toString().toLowerCase();
		if (!((result.equals("before"))
				||(result.equals("after"))
				||(result.equals("around")))){
			result = "before";
		}
		return result;
	}


	public static String generateSymmIntertypes(org.eclipse.uml2.uml.Package pack)
	{
		
		String baseTheme = getBaseThemeName(pack.getModel());
		String ret = "";
		for (Element e : pack.allOwnedElements())
		{
			if (e instanceof org.eclipse.uml2.uml.Class)
			{
				org.eclipse.uml2.uml.Class myClass = (org.eclipse.uml2.uml.Class) e;
				if (myClass.getOperations().size() == 0 && myClass.getAttributes().size() == 0)
					continue;
				ret += "\r\npublic aspect ";
				ret += myClass.getName();
				ret += "\r\n{\r\n";
				for (Property attr : myClass.getAllAttributes())
				{
					ret += "\t";
					ret += attr.getVisibility().toString();
					ret += " ";
					if (attr.getType() == null)
						ret += "void";
					else
						ret += attr.getType().getName();
					ret += " ";
					ret += baseTheme;
					ret += ".";
					ret += myClass.getName() + "." + attr.getName() + ";\r\n";
				}
				ret += "\r\n\r\n";
				for (Operation op : myClass.getOperations())
				{
					ret += "\t";
					ret += op.getVisibility().toString();
					ret += " ";
					if (op.getType() == null)
						ret += "void";
					else
						ret += op.getType().getName();
					ret += " ";
					ret += baseTheme;
					ret += ".";
					ret += myClass.getName() + "." + op.getName();
					ret += "(";
					for (Parameter p : op.getOwnedParameters())
					{
						if (p.getDirection() == ParameterDirectionKind.RETURN_LITERAL)
							continue;
						ret += p.getType().getName() + " " + p.getName();
						ret += ", ";
						
					}
					if (ret.endsWith(", "))
						ret = ret.substring(0, ret.length() - 2);
					ret += ");\r\n";
					
				}
				ret += "}\r\n";
			}
		}
		return ret;
	}
	
	public static String getBaseThemeName(org.eclipse.uml2.uml.Package pack)
	{
		org.eclipse.uml2.uml.Profile themeUMLProfile = pack.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoMerge = themeUMLProfile.getOwnedStereotype("merge"); 
		
		for (Element e : pack.allOwnedElements())
		{
			if (e instanceof Dependency)
			{
				Dependency dep = (Dependency) e;
				if (dep.isStereotypeApplied(stereoMerge))
				{
					if (dep.getValue(stereoMerge, "themeName").toString().equals(""))
						continue;
					else
						return dep.getValue(stereoMerge, "themeName").toString();
				}
				
			}
		}
		return "";
	}
	
	public static EList getStubs(org.eclipse.uml2.uml.Package pack)
	{
		EList<String> stubs = new UniqueEList();
		org.eclipse.uml2.uml.Profile themeUMLProfile = pack.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 
		
		for (Element e : pack.allOwnedElements())
		{
			if (e instanceof Package)
			{
				Package p = (Package) e;
				if (p.isStereotypeApplied(stereoTheme))
				{
					if (p.getValue(stereoTheme, "template") != null)
					{
						for (Element elem : p.allOwnedElements())
						{
							if (elem instanceof Class && !(elem instanceof Interaction))
							{
								Class c = (Class) elem;
								stubs.add(c.getName());
							}
						}
					}
				}
			}
		}
		return stubs;
	}
	
	public static EList getTemplatedClasses(org.eclipse.uml2.uml.Package pack)
	{
		/** Currently only returns one class = because only one template currently supported */
		EList<Class> list = new UniqueEList();
		org.eclipse.uml2.uml.Profile themeUMLProfile = pack.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 
		
		String templateString = (String) pack.getValue(stereoTheme, "template");
		templateString = templateString.substring(1, templateString.length() - 1);
		
		int rem = templateString.indexOf(".");
		templateString = templateString.substring(0, rem);
		for (Element e : pack.allOwnedElements())
		{
			if (e instanceof Class)
			{
				Class c = (Class) e;
				if (c.getName().equals(templateString))
					list.add(c);
			}
		}
		return list;
	}
	
	public static EList getNonTemplatedClasses(org.eclipse.uml2.uml.Package pack)
	{
		/** Currently only returns one class = because only one template currently supported */
		EList<Class> list = new UniqueEList();
		org.eclipse.uml2.uml.Profile themeUMLProfile = pack.getModel().getAppliedProfile("ThemeUML");
		org.eclipse.uml2.uml.Stereotype stereoTheme = themeUMLProfile.getOwnedStereotype("theme"); 
		
		String templateString = (String) pack.getValue(stereoTheme, "template");
		templateString = templateString.substring(1, templateString.length() - 1);
		
		int rem = templateString.indexOf(".");
		templateString = templateString.substring(0, rem);
		for (Element e : pack.allOwnedElements())
		{
			if (e instanceof Class && !(e instanceof Interaction))
			{
				Class c = (Class) e;
				if (!c.getName().equals(templateString))
					list.add(c);
			}
		}
		return list;
	}
	
	public static boolean isDirectionReturn(org.eclipse.uml2.uml.Parameter param)
	{
		return (param.getDirection() == ParameterDirectionKind.RETURN_LITERAL);
		
	}
	
	public static String generateAdvice(org.eclipse.uml2.uml.Package theme)
	{
		try
		{
		String templateClass = getTemplateClass(theme);
		String templateOp = getTemplateOp(theme);
		templateOp = templateOp.substring(0, templateOp.indexOf("("));
		Operation generateOp = null;
		Operation doOp = null;
		String code = "";
		for (Element e : theme.allOwnedElements())
		{
			if (e instanceof Class)
			{
				Class c = (Class) e;
				if (c.getName().equals(templateClass))
				{
					for (Operation o : c.getAllOperations())
					{
						if (o.getName().equals(templateOp))
						{
							generateOp = o;
						}
						else if (o.getName().equals("_do_" + templateOp))
						{
							doOp = o;
						}
					}
				}
			}
		}
		if (generateOp == null || doOp == null)
			return "";
		
	
		Interaction generateInter = null;
		// Get interaction
		for (Element e : theme.allOwnedElements())
		{
			if (e instanceof Interaction)
			{
				generateInter = (Interaction) e;
			}
		}
		if (generateInter == null)
			return "";
		
		LinkedList<Message> messageStack = new LinkedList<Message>();
		String typeOfAdvice = "";
		for (Message m : generateInter.getMessages())
		{
			if (m.getSignature() instanceof Operation)
			{
				messageStack.add(m);
			}
		}
		if(messageStack.get(1).getSignature() == doOp)
			typeOfAdvice = "after";
		else if (messageStack.get(messageStack.size()-1).getSignature() == doOp)
			typeOfAdvice = "before";
		else
			typeOfAdvice = "void around";
		
		code += typeOfAdvice + "(";
		//add the parameters here
		String arguments = "";
		String arguments1 = "";
		for(int i =0; i<generateOp.getOwnedParameters().size(); i++)
		{
			Parameter p = generateOp.getOwnedParameters().get(i);
			//generateOp.getReturnResult()!=p
			if(p.getDirection().getName().equals("in"))
			{
				arguments += p.getType().getName() + " " + p.getName() + ", ";
				arguments1 += p.getName() + ", ";
			}
		}
		if(arguments.length()>0)
			arguments = arguments.substring(0, arguments.lastIndexOf(","));
		if(arguments1.length()>0)
			arguments1 = arguments1.substring(0, arguments1.lastIndexOf(","));

		code += arguments + ")";
		if(typeOfAdvice.equals("after"))
		{
			code += " returning(Object data)";
		}
		code += ": ";
		code += generateOp.getName();
		code += "(" + arguments1 +  ")";
		code += " {\r\n\t";
		
		
		//this piece of code does NOT work!
		for (Lifeline lifeLine : generateInter.getLifelines()) {
			for (Message m : messageStack) {
				MessageEnd re = m.getReceiveEvent();				
				MessageEnd se = m.getSendEvent();
				if (m.getSignature() == doOp)
				{
					if (typeOfAdvice.equals("void around"))
						code += "\tproceed();\n\t";
					continue;
				}
				if (se instanceof Gate)
				{
					if (m.getSignature() != generateOp)
						return "";
				}
				else if (re instanceof MessageOccurrenceSpecification) {
					MessageOccurrenceSpecification mos = (MessageOccurrenceSpecification) re;
					MessageOccurrenceSpecification mos2 = (MessageOccurrenceSpecification) se;
					//workaround as mos.getCovereds().contains(lifeLine) doesn't work - Aram Hovsepyan
					boolean covered = false;
					for(Lifeline l : mos2.getCovereds())
					{
						//couldn't come up with anything better for the moment
						if(l.toString().equals(lifeLine.toString()))
							covered = true;
					}
					if (covered) {
						//The event is of type calleventimpl and not SendOperationEvent - Aram H.
						if (mos.getEvent() instanceof CallEventImpl) { 
							//SendOperationEvent soe = (SendOperationEvent) mos.getEvent();
							CallEventImpl soe = (CallEventImpl) mos.getEvent();
							if (re instanceof MessageOccurrenceSpecification) {
								Lifeline destination = mos.getCovereds().get(0);
								if (soe.getOperation() != null) {
									// if operation has a return type then the
									// first argument should be it
									if (soe.getOperation().getReturnResult() != null) {
										if (m.getArguments().size() > 0) {
											ValueSpecification vs = m
													.getArguments().get(0);
											/*
											if (vs instanceof LiteralString) {
												code += ((LiteralString) vs)
														.stringValue();
												code += " = ";
											}*/
										}

									}
									code += "\t";
									if (destination == lifeLine) {
										code += "this.";
									} 
									else
									{
										code += destination.getRepresents().getName();
										if(destination.getRepresents().getName()=="")
											code += destination.getRepresents().getType().getName().toLowerCase();
										code += ".";	
									}
									code += soe.getOperation().getName();
									code += "(";
									for(int x=0; x<soe.getOperation().getOwnedElements().size(); x++)
									{
										Parameter p = (Parameter) soe.getOperation().getOwnedElements().get(x);
										if(!p.getDirection().getName().equals("return"))
										{
											if(!code.endsWith("("))
												code += ", ";
											code += p.getName();
										}
									}
									code += ");\n\t";

								}

							}
							else if (re instanceof Gate)
							{
								for (ValueSpecification vx : m
										.getArguments()) {
									if (vx instanceof OpaqueExpression) {
											code += "return ";
										code += ((OpaqueExpression) vx)
												.getBodies().get(0);
										code += ";";
									}
									}
							}
						}
					}
				}

			}
			return code + "\r\n\t}\n";
			//return code;
		}
		return "";
		}
		catch (NullPointerException npe)
		{
			npe.printStackTrace();
			return "";
		}
	}
	
	public static String getPrefix(org.eclipse.uml2.uml.Property prop)
	{
		
		String ret = "";
		Element e = prop.getOwner();
		if (e != null && e instanceof NamedElement)
		{
			NamedElement ne = (NamedElement) e;
			if (prop.getModel() == null)
				return "";
			ret += getBaseThemeName(prop.getModel()).toLowerCase() + ".";
			ret += ne.getName();
			
		}
		return ret + ".";
	}
	public static String getPrefix(org.eclipse.uml2.uml.Operation prop)
	{
		
		String ret = "";
		Element e = prop.getOwner();
		if (e != null && e instanceof NamedElement)
		{
			NamedElement ne = (NamedElement) e;
			if (prop.getModel() == null)
				return "";
			ret += getBaseThemeName(prop.getModel()).toLowerCase() + ".";
			ret += ne.getName();
			
		}
		return ret + ".";
	}
	
	public static LinkedList<EObject> doMerge(org.eclipse.uml2.uml.Model model)
	{
		Profile themeProfile = model.getAppliedProfile("ThemeUML");
		Stereotype stereoMerge = themeProfile.getOwnedStereotype("merge");
		LinkedList<Dependency> merges = new LinkedList<Dependency>();
		// Find all merges
		for (Element element : model.allOwnedElements())
		{	
			if (element.isStereotypeApplied(stereoMerge)
					&& element instanceof Dependency)
			{
				merges.add((Dependency) element);
			}
		}
		
		Dependency mainMergeDependency = null;
		// Find main merge dependency (the one with the stereotypes applied)
		for (Dependency dep : merges)
		{
			if ((dep.getSources().get(0) instanceof org.eclipse.uml2.uml.Package)
					&& (dep.getTargets().get(0) instanceof org.eclipse.uml2.uml.Package))
				mainMergeDependency = dep;
		}

		LinkedList<org.eclipse.uml2.uml.Package> themes = new LinkedList<org.eclipse.uml2.uml.Package>();
		themes.add((org.eclipse.uml2.uml.Package) mainMergeDependency.getSources().get(0));
		themes.add((org.eclipse.uml2.uml.Package) mainMergeDependency.getTargets().get(0));
		for (Dependency dep : merges)
		{
			if (dep.getSources().get(0) == mainMergeDependency)
				themes.add((org.eclipse.uml2.uml.Package) dep.getTargets().get(0));
		}

		org.eclipse.uml2.uml.Package emptyP =  UMLFactory.eINSTANCE.createPackage();
		themes.add(0, emptyP);
		while (themes.size() > 1)
		{
			org.eclipse.uml2.uml.Package p1 = themes.get(0);
			org.eclipse.uml2.uml.Package p2 = themes.get(1);
			themes.remove(p1);
			themes.remove(p2);
			PackageMerge pm = p1.createPackageMerge(p2);
			themeMerger.merge(p1, null, null, null);
			themes.add(0, p1);
		}
		LinkedList<EObject> mo = themeMerger.matchedObjects;
		/*for(EObject m: mo)
		{
			System.out.println(m.toString());
		}*/
		return mo;
	}
	
	public static boolean isUMLClass(Object o)
	{
		if (o instanceof Class)
		{
			return true;
		}
		return false;
	}
	
	public static boolean isClass(NamedElement name)
	{
		if (name == null)
			return false;
		if (name.getModel() == null)
			return false;
		for (Element e : name.getModel().allOwnedElements())
		{
			if (e instanceof Class)
			{
				
				Class c = (Class) e;
				
				if (c.getName().equals(name.getName()))
				{
					
					return true; 
				}
			}
			else if (e instanceof Interface)
			{
				
				Interface c = (Interface) e;
				
				if (c.getName().equals(name.getName()))
				{
					
					return true; 
				}
			}
			else if (e instanceof Enumeration)
			{
				
				Enumeration c = (Enumeration) e;
				
				if (c.getName().equals(name.getName()))
				{
					
					return true; 
				}
			}
			
		}
		return false;
	}
	
	public static boolean isConflicting(org.eclipse.uml2.uml.Class c)
	{
		for (EObject e : themeMerger.matchedObjects)
		{
			if (e instanceof Class)
			{
				org.eclipse.uml2.uml.Class c2 = (org.eclipse.uml2.uml.Class) e;
				
				if (c.getName().equals(c2.getName()))
					return true;
			}
		}
		return false;
	}
	public static boolean isConflicting(String s)
	{
		for (EObject e : themeMerger.matchedObjects)
		{
			if (e instanceof Class)
			{
				org.eclipse.uml2.uml.Class c2 = (org.eclipse.uml2.uml.Class) e;
				
				if (c2.getName().equals(s))
					return true;
			}
		}
		return false;
	}
	
	public static Class getConflict(String s)
	{
		for (EObject e : themeMerger.matchedObjects)
		{
			if (e instanceof Class)
			{
				org.eclipse.uml2.uml.Class c2 = (org.eclipse.uml2.uml.Class) e;
				
				if (c2.getName().equals(s))
					return c2;
			}
		}
		return null;
	}

	public static String getTypeModifier(org.eclipse.uml2.uml.Property param)
	{
		 String result = "";
		 final Stereotype stereo = param.getAppliedStereotype("MagicDraw Profile::typeModifier");
		 if ((stereo != null) && "[]".equals(param.getValue(stereo, "typeModifier"))) {
		     result = "[]";
		 }
		 else if((stereo != null) && "*".equals(param.getValue(stereo, "typeModifier"))) {
		     result = "*";
		 } 
		 return result;
	}
	
	public static String getTypeName(org.eclipse.uml2.uml.Parameter param)
	{
		if(param.getType()==null)
			return "Object";
		 String result = param.getType().getName();
		 if(result.equals("") || result.equals("OclAny"))
			 return "Object";
		 return result;
	}
}
