package helpers;


import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.eclipse.emf.common.util.BasicEMap;
import org.eclipse.emf.common.util.DiagnosticChain;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.UniqueEList;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.uml2.common.util.CacheAdapter;
import org.eclipse.uml2.common.util.UML2Util;
import org.eclipse.uml2.common.util.UML2Util.EClassMatcher;
import org.eclipse.uml2.common.util.UML2Util.EStructuralFeatureMatcher;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.DirectedRelationship;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.EnumerationLiteral;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.TypedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.eclipse.uml2.uml.util.UMLUtil;
import org.eclipse.uml2.uml.util.UMLUtil.PackageMerger;


public class ThemeMerger extends PackageMerger
{

	private static final long serialVersionUID = 8167411441655270843L;

	protected boolean doMatch = true;

	public LinkedList<EObject> matchedObjects = new LinkedList<EObject>();

	@Override
	public EObject copy(EObject eObject)
	{
		EObject copyEObject = super.copy(eObject);

		if (eObject != copyEObject)
		{
			List<EObject> mergedEObjects = resultingToMergedEObjectMap
					.get(copyEObject);

			if (mergedEObjects == null)
			{
				resultingToMergedEObjectMap
						.put(
								copyEObject,
								mergedEObjects = new UniqueEList.FastCompare<EObject>(
										1));
			}

			mergedEObjects.add(eObject);
		}

		return copyEObject;
	}

	@Override
	protected void copyAttribute(EAttribute eAttribute, EObject eObject,
			EObject copyEObject)
	{
		

		if (eObject != copyEObject)
		{
			if (copyEObject == receivingPackage)
				return;
			// Resolve conflicts the theme way before packagemerge has
			// the change to use its own transsformations.

			if (matchedObjects.contains(copyEObject))
			{
				
			}
			super.copyAttribute(eAttribute, eObject, copyEObject);

		}

	}

	@Override
	protected void copyReference(EReference reference, EObject object,
			EObject copyEObject)
	{
		super.copyReference(reference, object, copyEObject);
	}

	protected class BodyMatcher extends EStructuralFeatureMatcher
	{

		protected BodyMatcher(Comment comment)
		{
			super(comment, UMLPackage.Literals.COMMENT__BODY);
		}
	}

	protected class NameMatcher extends EStructuralFeatureMatcher
	{
		protected NameMatcher(NamedElement namedElement)
		{
			super(namedElement, UMLPackage.Literals.NAMED_ELEMENT__NAME);
		}
	}

	protected class KeyMatcher extends EClassMatcher
	{

		protected KeyMatcher(BasicEMap.Entry<String, String> entry)
		{
			super((EObject) entry);
		}

		@SuppressWarnings("unchecked")
		@Override
		public boolean matches(EObject otherEObject)
		{
			return super.matches(otherEObject)
					&& UML2Util.safeEquals(
							((BasicEMap.Entry<String, String>) eObject)
									.getKey(),
							((BasicEMap.Entry<String, String>) otherEObject)
									.getKey());
		}
	}

	protected class ResultingQNameMatcher extends EClassMatcher
	{

		protected ResultingQNameMatcher(EObject eObject)
		{
			super(eObject);
		}

		@Override
		public boolean matches(EObject otherEObject)
		{
			return super.matches(otherEObject)
					&& UML2Util.safeEquals(getResultingQName(eObject),
							getResultingQName(otherEObject));
		}
	}

	protected class StringValueMatcher extends EClassMatcher
	{

		protected StringValueMatcher(ValueSpecification valueSpecification)
		{
			super(valueSpecification);
		}

		@Override
		public boolean matches(EObject otherEObject)
		{
			return super.matches(otherEObject)
					&& UML2Util.safeEquals(((ValueSpecification) eObject)
							.stringValue(), ((ValueSpecification) otherEObject)
							.stringValue());
		}
	}

	protected class SourceMatcher extends EStructuralFeatureMatcher
	{

		protected SourceMatcher(EAnnotation eAnnotation)
		{
			super(eAnnotation, EcorePackage.Literals.EANNOTATION__SOURCE);
		}
	}

	protected class TypeMatcher extends NameMatcher
	{

		protected TypeMatcher(TypedElement typedElement)
		{
			super(typedElement);
		}

		@Override
		public boolean matches(EObject otherEObject)
		{

			if (super.matches(otherEObject))
			{
				Type type = ((TypedElement) eObject).getType();
				Type otherType = ((TypedElement) otherEObject).getType();

				return new ResultingQNameMatcher(type).matches(otherType)
						|| (type instanceof Classifier
								&& otherType instanceof Classifier && 
								(UML2Util.findEObject(((Classifier) type).allParents(),
										new ResultingQNameMatcher(otherType)) != null || 
								 UML2Util.findEObject(((Classifier) otherType)
										.allParents(),
										new ResultingQNameMatcher(type)) != null));
			}

			return false;
		}
	}
	protected static EObject applyStereotype(Element element, EClass definition)
	{
		EObject stereotypeApplication = EcoreUtil.create(definition);

		CacheAdapter.INSTANCE.adapt(stereotypeApplication);

		Resource eResource = element.eResource();

		if (eResource != null)
		{
			eResource.getContents().add(stereotypeApplication);
		}

		UMLUtil.setBaseElement(stereotypeApplication, element);

		return stereotypeApplication;
	}

	@Override
	protected EObject createCopy(EObject eObject)
	{
		return new UMLSwitch<EObject>()
		{

			@Override
			public EObject caseAssociation(Association association)
			{
				Association matchingAssociation = (Association) UML2Util
						.findEObject(getMatchCandidates(association),
								new NameMatcher(association)
								{

									@Override
									public boolean matches(EObject otherEObject)
									{

										if (super.matches(otherEObject))
										{
											otherEObject = getPreviouslyMergedEObject(otherEObject);

											List<Property> memberEnds = ((Association) eObject)
													.getMemberEnds();
											List<Property> otherMemberEnds = ((Association) otherEObject)
													.getMemberEnds();

											if (memberEnds.size() == otherMemberEnds
													.size())
											{

												for (Property memberEnd : memberEnds)
												{

													if (UML2Util.findEObject(
															otherMemberEnds,
															new TypeMatcher(
																	memberEnd)) == null)
													{

														return false;
													}
												}

												return true;
											}

										}

										return false;
									}
								});

				if (matchingAssociation != null)
					matchedObjects.add(matchingAssociation);
				return matchingAssociation == null ? super
						.caseAssociation(association) : matchingAssociation;
			}

			@Override
			public EObject caseClass(org.eclipse.uml2.uml.Class class_)
			{
				org.eclipse.uml2.uml.Class matchingClass = (org.eclipse.uml2.uml.Class) UML2Util
						.findEObject(getMatchCandidates(class_),
								new NameMatcher(class_));

				if (matchingClass != null)
					matchedObjects.add(matchingClass);

				return matchingClass == null
						? super.caseClass(class_)
						: matchingClass;
			}

			@Override
			public EObject caseComment(Comment comment)
			{
				Comment matchingComment = (Comment) UML2Util.findEObject(
						getMatchCandidates(comment), new BodyMatcher(comment));

				if (matchingComment != null)
					matchedObjects.add(matchingComment);

				return matchingComment == null
						? super.caseComment(comment)
						: matchingComment;
			}

			@Override
			public EObject caseConstraint(Constraint constraint)
			{
				Constraint matchingConstraint = (Constraint) UML2Util
						.findEObject(getMatchCandidates(constraint),
								new NameMatcher(constraint));

				if (matchingConstraint != null)
					matchedObjects.add(matchingConstraint);

				return matchingConstraint == null ? super
						.caseConstraint(constraint) : matchingConstraint;
			}

			@Override
			public EObject caseDataType(DataType dataType)
			{
				DataType matchingDataType = (DataType) UML2Util
						.findEObject(getMatchCandidates(dataType),
								new NameMatcher(dataType));

				if (matchingDataType != null)
					matchedObjects.add(matchingDataType);

				return matchingDataType == null
						? super.caseDataType(dataType)
						: matchingDataType;
			}

			@Override
			public EObject caseDirectedRelationship(
					DirectedRelationship directedRelationship)
			{

				if (mergedPackages.containsAll(directedRelationship
						.getTargets()))
				{

					return directedRelationship;
				}

				DirectedRelationship matchingDirectedRelationship = (DirectedRelationship) UML2Util
						.findEObject(getMatchCandidates(directedRelationship),
								new EClassMatcher(directedRelationship)
								{

									@Override
									public boolean matches(EObject otherEObject)
									{

										if (super.matches(otherEObject))
										{
											otherEObject = getPreviouslyMergedEObject(otherEObject);

											List<Element> targets = ((DirectedRelationship) eObject)
													.getTargets();
											List<Element> otherTargets = ((DirectedRelationship) otherEObject)
													.getTargets();

											if (targets.size() == otherTargets
													.size())
											{

												for (Element target : targets)
												{

													if (UML2Util
															.findEObject(
																	otherTargets,
																	new ResultingQNameMatcher(
																			target)) == null)
													{

														return false;
													}
												}

												return true;
											}
										}

										return false;
									}
								});

				if (matchingDirectedRelationship != null)
					matchedObjects.add(matchingDirectedRelationship);
				return matchingDirectedRelationship == null
						? super.caseDirectedRelationship(directedRelationship)
						: matchingDirectedRelationship;
			}

			@Override
			public EObject caseEnumerationLiteral(
					EnumerationLiteral enumerationLiteral)
			{
				EnumerationLiteral matchingEnumerationLiteral = (EnumerationLiteral) UML2Util
						.findEObject(getMatchCandidates(enumerationLiteral),
								new NameMatcher(enumerationLiteral));

				if (matchingEnumerationLiteral != null)
					matchedObjects.add(matchingEnumerationLiteral);

				return matchingEnumerationLiteral == null
						? super.caseEnumerationLiteral(enumerationLiteral)
						: matchingEnumerationLiteral;
			}
			
			@Override
			public EObject caseInterface(org.eclipse.uml2.uml.Interface interface_)
			{
				org.eclipse.uml2.uml.Interface matchingInterface = (org.eclipse.uml2.uml.Interface) UML2Util
						.findEObject(getMatchCandidates(interface_),
								new NameMatcher(interface_));

				if (matchingInterface != null)
					matchedObjects.add(matchingInterface);

				return matchingInterface == null
						? super.caseInterface(interface_)
						: matchingInterface;
			}
			
			@Override
			public EObject caseOperation(Operation operation)
			{
				Operation matchingOperation = (Operation) UML2Util.findEObject(
						getMatchCandidates(operation), new NameMatcher(
								operation)
						{

							@Override
							public boolean matches(EObject otherEObject)
							{

								if (super.matches(otherEObject))
								{
									otherEObject = getPreviouslyMergedEObject(otherEObject);

									List<Parameter> ownedParameters = ((Operation) eObject)
											.getOwnedParameters();
									List<Parameter> otherOwnedParameters = ((Operation) otherEObject)
											.getOwnedParameters();

									if (ownedParameters.size() == otherOwnedParameters
											.size())
									{

										for (int i = 0; i < ownedParameters
												.size(); i++)
										{

											if (!new TypeMatcher(
													ownedParameters.get(i))
													.matches(otherOwnedParameters
															.get(i)))
											{

												return false;
											}
										}

										return true;
									}
								}

								return false;
							}
						});
				if (matchingOperation != null)
					matchedObjects.add(matchingOperation);
				
				
				return matchingOperation == null ? super
						.caseOperation(operation) : matchingOperation;
			}

			@Override
			public EObject casePackage(org.eclipse.uml2.uml.Package package_)
			{
				org.eclipse.uml2.uml.Package matchingPackage = null;

				if (mergedPackages.contains(package_))
				{
					matchingPackage = receivingPackage;
				}
				else
				{
					matchingPackage = (org.eclipse.uml2.uml.Package) UML2Util
							.findEObject(getMatchCandidates(package_),
									new NameMatcher(package_));
				}

				if (matchingPackage != null)
					matchedObjects.add(matchingPackage);

				return matchingPackage == null
						? super.casePackage(package_)
						: matchingPackage;
			}

			@Override
			public EObject caseParameter(Parameter parameter)
			{
				Parameter matchingParameter = (Parameter) UML2Util.findEObject(
						getMatchCandidates(parameter), new NameMatcher(
								parameter));

				if (matchingParameter != null)
					matchedObjects.add(matchingParameter);

				return matchingParameter == null ? super
						.caseParameter(parameter) : matchingParameter;
			}

			@Override
			public EObject caseProperty(Property property)
			{
				
				Property matchingProperty = (Property) UML2Util
						.findEObject(getMatchCandidates(property),
								new NameMatcher(property));
				
				
				if (matchingProperty != null)
					matchedObjects.add(matchingProperty);

				return matchingProperty == null
						? super.caseProperty(property)
						: matchingProperty;
			}

			@Override
			public EObject defaultCase(EObject eObject)
			{
				Element baseElement = UMLUtil.getBaseElement(eObject);

				return baseElement == null
						? ThemeMerger.super.createCopy(eObject)
						: applyStereotype((Element) get(baseElement),
								getTarget(eObject.eClass()));
			}

			@Override
			protected EObject doSwitch(EClass theEClass, EObject theEObject)
			{

				if (theEClass.eContainer() != modelPackage)
				{

					if (theEClass == EcorePackage.Literals.EANNOTATION)
					{
						EAnnotation matchingEAnnotation = (EAnnotation) UML2Util
								.findEObject(getMatchCandidates(theEObject),
										new SourceMatcher(
												(EAnnotation) theEObject));

						if (matchingEAnnotation != null)
						{
							return matchingEAnnotation;
						}
					}
					else if (theEClass == EcorePackage.Literals.ESTRING_TO_STRING_MAP_ENTRY)
					{
						@SuppressWarnings("unchecked")
						BasicEMap.Entry<String, String> matchingEntry = (BasicEMap.Entry<String, String>) UML2Util
								.findEObject(
										getMatchCandidates(theEObject),
										new KeyMatcher(
												(BasicEMap.Entry<String, String>) theEObject));

						if (matchingEntry != null)
						{
							return (EObject) matchingEntry;
						}
					}
					else if (getStereotype(theEClass, theEObject) != null)
					{
						EObject matchingEObject = UML2Util.findEObject(
								getMatchCandidates(theEObject),
								new EClassMatcher(theEObject));

						if (matchingEObject != null)
						{
							return matchingEObject;
						}
					}
				}

				return super.doSwitch(theEClass, theEObject);
			}
		}.doSwitch(eObject);
	}

	protected static Stereotype getStereotype(EClass definition)
	{
		return getStereotype(definition, null);
	}

	protected static Stereotype getStereotype(EClass definition,
			EObject stereotypeApplication)
	{
		NamedElement namedElement = getNamedElement(definition,
				stereotypeApplication);
		return namedElement instanceof Stereotype
				? (Stereotype) namedElement
				: null;
	}

	protected static NamedElement getNamedElement(ENamedElement definition)
	{
		return getNamedElement(definition, null);
	}

	protected static NamedElement getNamedElement(ENamedElement definition,
			EObject context)
	{

		if (definition instanceof EClassifier)
		{
			EAnnotation eAnnotation = definition
					.getEAnnotation(UMLPackage.eNS_URI);

			if (eAnnotation != null)
			{
				EList<EObject> references = eAnnotation.getReferences();

				if (!references.isEmpty())
				{
					EObject reference = references.get(0);

					if (reference instanceof Classifier)
					{
						return (NamedElement) reference;
					}
				}
			}

			org.eclipse.uml2.uml.Package package_ = (org.eclipse.uml2.uml.Package) getNamedElement(
					((EClassifier) definition).getEPackage(), context);

			if (package_ != null)
			{
				String name = definition.getName();

				for (Type ownedType : package_.getOwnedTypes())
				{

					if (UML2Util.safeEquals(UML2Util
							.getValidJavaIdentifier(ownedType.getName()), name))
					{

						return ownedType;
					}
				}
			}

			return null;
		}
		else if (definition instanceof EStructuralFeature)
		{
			org.eclipse.uml2.uml.Class class_ = (org.eclipse.uml2.uml.Class) getNamedElement(
					((EStructuralFeature) definition).getEContainingClass(),
					context);

			if (class_ != null)
			{
				String name = definition.getName();

				for (Property ownedAttribute : class_.getOwnedAttributes())
				{

					if (UML2Util.safeEquals(UML2Util
							.getValidJavaIdentifier(ownedAttribute.getName()),
							name))
					{

						return ownedAttribute;
					}
				}
			}

			return null;
		}
		else if (definition instanceof EEnumLiteral)
		{
			Enumeration enumeration = (Enumeration) getNamedElement(
					((EEnumLiteral) definition).getEEnum(), context);
			return enumeration == null ? null : enumeration
					.getOwnedLiteral(definition.getName());
		}
		else if (definition instanceof EPackage)
		{
			return UMLUtil.getProfile((EPackage) definition, context);
		}
		else
		{
			return null;
		}
	}

	@Override
	public /*java.util.Map<org.eclipse.emf.ecore.EObject,java.util.List<org.eclipse.emf.ecore.EObject>>*/ void merge(Package package_, Map<String, String> options,
			DiagnosticChain diagnostics, Map<Object, Object> context)
	{
		/*return*/ super.merge(package_, options, diagnostics, context);
	}


	private LinkedList<String> stackConstructs(String construct)
	{
		LinkedList<String> r = new LinkedList<String>();

		StringTokenizer st = new StringTokenizer(construct, ".");
		while (st.hasMoreTokens())
			r.add(st.nextToken());
		return r;
	}

	/**
	 * this function is not used at all!*/
	public boolean matches(Operation o1, Operation o2)
	{
		System.out.println("invoked");
		List<Parameter> ownedParameters = o1.getOwnedParameters();
		List<Parameter> otherOwnedParameters = o2.getOwnedParameters();
		if (ownedParameters.size() == otherOwnedParameters.size())
		{
			for (int i = 0; i < ownedParameters.size(); i++)
			{

				if (!new TypeMatcher(ownedParameters.get(i)).matches(otherOwnedParameters.get(i)))
				{

					return false;
				}
			}

			return true;
		}
		return false;
	}

}
