package library.aspects.context;

import library.interfaces.Checkpointing;
import library.interfaces.Checkpointable;
import library.interfaces.Context;
import library.interfaces.Tracing;
import library.interfaces.Traceable;
import library.util.AccessTypes;
import java.util.Set;

import library.util.Debugger;

public aspect CheckpointingAspect 
{	
	//declare parents: Checkpointing implements Tracing;
	
	// Finalization: Simply discard all the established checkpoints
	public void Checkpointing.finalizeCheckpoints()
	{
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,this,"The checkpoints of the context will be finalized.",2);
		
		Set<Object> modifiedObjects = this.getModifiedObjects();
		
		for(Object obj: modifiedObjects)
		{
			Checkpointable checkpointable = (Checkpointable) obj;
			
			checkpointable.discardCheckPoint();
		}
		
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,this,"The checkpoints of the context have been finalized.",2);
	}
	
	pointcut autoCheckPointingContextCreation(Checkpointing autoCheckPointingContext):
    	target(autoCheckPointingContext) && execution(public Checkpointing+.new(..)) ;
	
	after(Checkpointing autoCheckPointingContext) : autoCheckPointingContextCreation(autoCheckPointingContext)
	{
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,autoCheckPointingContext,"Constructor Execution call (after) has been intercepted.",2);
		
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,autoCheckPointingContext,"Checkpointing service inititated.",1);
		
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,autoCheckPointingContext,"Constructor Execution call (after) has been returned.",2);
	}
	
	pointcut autoCheckPointingContextDestruction(Checkpointing autoCheckPointingContext):
		this(autoCheckPointingContext) && call(public void Context.inactivateContext(..));
	
	before(Checkpointing autoCheckPointingContext) : autoCheckPointingContextDestruction(autoCheckPointingContext)
	{	
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,autoCheckPointingContext,"Inactivation call (before) has been intercepted.",2);
		
		autoCheckPointingContext.finalizeCheckpoints();
		
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,autoCheckPointingContext,"Inactivation call (before) has been returned.",2);
	}
	
	pointcut memberPerformingWork(Checkpointing autoCheckPointingContext) : 
		call(public * Context.participantPerformsWork(..)) && target(autoCheckPointingContext);
	
	before(Checkpointing autoCheckPointingContext): memberPerformingWork(autoCheckPointingContext)
	{
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,autoCheckPointingContext,"The notification (before) call has been intercepted.",2);
		
		Object[] allArguments = thisJoinPoint.getArgs();
		
		Object accessedObject = allArguments[1];
		String methodName = (String) allArguments[2];
		
		// We may assume that the object is Traceable since Checkpointing depends on Tracing
		String accessType = ((Traceable) accessedObject).getAccessTypeOfMethod(methodName);
		String objectName = ((Traceable) accessedObject).getMyName();
		
		if(accessedObject instanceof Checkpointable)
		{
			Checkpointable checkpointableObject = (Checkpointable) accessedObject;
			
			if(!accessType.equals(AccessTypes.READ))
			{
				if(!autoCheckPointingContext.hasModifiedObject(objectName))
				{
					Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,autoCheckPointingContext,"A snap-shot of the object is requested.",1);
					
					checkpointableObject.establishCheckPoint();
				
				}
				else
				{
					//System.out.println("{CheckPointingAspect} There was already a checkpoint established for this object!!");
				}
			}
			else
			{
				//System.out.println("{CheckPointingAspect} It was a read access, so autocheckpointing did not establish any checkpoints!");
			}
		}
		
		else
		{
			//System.out.println("{CheckPointingAspect} Accessed Object is not Checkpointable");
		}
		
		Debugger.printMessage("Checkpointing",Debugger.CONTEXT_ASPECT,autoCheckPointingContext,"The notification call (before) has been released.",2);
	}
}
