package library.aspects.context;

import library.interfaces.Nested;
import library.interfaces.Checkpointing;
import library.interfaces.CheckpointingNested;
import library.interfaces.Checkpointable;

import java.util.Set;

import library.util.Debugger;

public aspect CheckpointingNestedAspect
{
	//declare parents: CheckpointingNested implements /*Nested,*/ Checkpointing;
	
	// Only discard the checkpoints that also exist in the parent...
	public void CheckpointingNested.finalizeNestedCheckpoints()
	{	
		CheckpointingNested parentContext = (CheckpointingNested) this.getParentContext();
		
		Set<Object> modifiedObjects = this.getModifiedObjects();
		
		for(Object object : modifiedObjects)
		{
			// If the parent has already established a checkpoint, then go ahead and discard child's.
			if(parentContext.hasModifiedObject(object))
			{
				Checkpointable checkpointable = (Checkpointable) object;
				
				checkpointable.discardCheckPoint();
			}	
		}
	}
	
	pointcut checkpointFinalization(CheckpointingNested context):
		call(public * finalizeCheckpoints(..)) && target(context);
	
	Object around(CheckpointingNested context) : checkpointFinalization(context)
	{
		Debugger.printMessage("CheckpointingNested",Debugger.CONTEXT_ASPECT,context,"The checkpoint finalization call has been intercepted.",2);
		
		if(!context.hasAParent())
		{
			Debugger.printMessage("CheckpointingNested",Debugger.CONTEXT_ASPECT,context,"The checkpoint finalization call has been released.",2);
			
			return proceed(context);
		}
		else
		{
			Debugger.printMessage("CheckpointingNested",Debugger.CONTEXT_ASPECT,context,"The finalization will be carried out along with nesting rules.",1);
			
			context.finalizeNestedCheckpoints();
			
			Debugger.printMessage("CheckpointingNested",Debugger.CONTEXT_ASPECT,context,"The checkpoint finalization call has been released.",2);
			
			return null;
		}	
	}
}
