package library.aspects.context;

import library.interfaces.Nested;
import library.interfaces.Deferring;
import library.interfaces.DeferringNested;
import library.interfaces.VersionedContextAware;

import java.util.Set;

import library.util.Debugger;

public aspect DeferringNestedAspect
{
	//declare parents: DeferringNested implements /*Nested, */Deferring;
	
	// Make sure that the parent can see the child's version
	public void DeferringNested.finalizeNestedVersions()
	{	
		DeferringNested parentContext = (DeferringNested) this.getParentContext();
		
		Set<Object> modifiedObjects = this.getModifiedObjects();
		
		for(Object object : modifiedObjects)
		{
			VersionedContextAware versioned = (VersionedContextAware) object;
	
			// Replace the parent's version with the child's...
			if(parentContext.hasModifiedObject(object))
			{
				versioned.unregisterContext(parentContext);
			
				int childVersion = versioned.lookUpVersionForContext(this);

				versioned.registerContextForVersion(parentContext,childVersion);

				versioned.unregisterContext(this);			
			}
			
			// The parent should have the child's version in its view
			else
			{
				int childVersion = versioned.lookUpVersionForContext(this);
	
				versioned.registerContextForVersion(parentContext,childVersion);
			}	
		}
	}
	
	pointcut versionFinalization(DeferringNested context):
		call(public * finalizeVersions(..)) && target(context);
	
	Object around(DeferringNested context) : versionFinalization(context)
	{
		Debugger.printMessage("DeferringNested",Debugger.CONTEXT_ASPECT,context,"The version finalization call has been intercepted.",2);
		
		if(!context.hasAParent())
		{
			Debugger.printMessage("DeferringNested",Debugger.CONTEXT_ASPECT,context,"The version finalization call has been released.",2);
			
			return proceed(context);
		}
		else
		{
			Debugger.printMessage("DeferringNested",Debugger.CONTEXT_ASPECT,context,"The finalization will be carried out along with nesting rules.",1);
			
			context.finalizeNestedVersions();
			
			Debugger.printMessage("DeferringNested",Debugger.CONTEXT_ASPECT,context,"The version finalization call has been released.",2);
			
			return null;
		}	
	}
}

