package library.aspects.context;

import library.interfaces.EntrySynchronizing;
import library.interfaces.Collaborative;

import library.util.Debugger;

public aspect EntrySynchronizingAspect introduces EntrySynchronizing
{	
	declare parents : EntrySynchronizing implements Collaborative;

	private int EntrySynchronizing.numberOfWaitingParticipants = 0;
	private int EntrySynchronizing.minimumMemberCount = 0;
	
	private boolean EntrySynchronizing.entrySynchronizationActive = false;
	
	private Object EntrySynchronizing.entryCondition = new Object();
	
	public boolean EntrySynchronizing.getEntrySynchronizationActive()
	{
		return entrySynchronizationActive;
	}
	
	public void EntrySynchronizing.setEntrySynchronizationActive(boolean value)
	{
		entrySynchronizationActive = value;
	}
	
	public void EntrySynchronizing.setMinimumMemberCount(int newMinimum)
	{
		minimumMemberCount = newMinimum;
		entrySynchronizationActive = true;
		
		Debugger.printMessage("EntrySynchronizing",Debugger.CONTEXT_ASPECT,this,"The minimum number of participants for synchronized entry " +
				"has been set as: " + newMinimum,1);
	}
	
	public int EntrySynchronizing.getMinimumMemberCount()
	{
		return minimumMemberCount;
	}
	
	public void EntrySynchronizing.decrementMinimumMemberCount()
	{
		minimumMemberCount--;
	}
	
	public void EntrySynchronizing.incrementNumberOfWaitingParticipants()
	{
		numberOfWaitingParticipants++;
	}
	
	public int EntrySynchronizing.getNumberOfWaitingParticipants()
	{
		return numberOfWaitingParticipants;
	}
	
	public Object EntrySynchronizing.getEntryConditionObject()
	{
		return entryCondition;
	}
	
	pointcut memberEntersTheContext(EntrySynchronizing synchEntryContext):
		call(public void Collaborative.addParticipant(..)) && target(synchEntryContext);
	
	after(EntrySynchronizing synchEntryContext) : memberEntersTheContext(synchEntryContext)
	{
		Debugger.printMessage("EntrySynchronizing",Debugger.CONTEXT_ASPECT,synchEntryContext,"The call for adding the participant -" + Thread.currentThread().getName() + "-" + " has been intercepted.",2);
		
		synchronized(synchEntryContext.getEntryConditionObject())
		{
			synchEntryContext.decrementMinimumMemberCount();
		
			if(synchEntryContext.getMinimumMemberCount() <= 0)
			{
				Debugger.printMessage("EntrySynchronizing",Debugger.CONTEXT_ASPECT,synchEntryContext,"The synchronization process has been completed." + " The participants may now start performing work.",1);
				
				synchEntryContext.getEntryConditionObject().notifyAll();
			}
			else
			{
				Debugger.printMessage("EntrySynchronizing",Debugger.CONTEXT_ASPECT,synchEntryContext,"The participant -" + Thread.currentThread().getName() + "-" + " is forced to wait at the entry of the context.",1);
				
				synchEntryContext.incrementNumberOfWaitingParticipants();
				
				try
				{
					synchEntryContext.getEntryConditionObject().wait();
				}
				catch(InterruptedException ex){}
			}
		}
		
		Debugger.printMessage("EntrySynchronizing",Debugger.CONTEXT_ASPECT,synchEntryContext,"The call for adding a participant has been released.",2);
	}
	
}
