package library.aspects.context;

import library.interfaces.ExitSynchronizing;
import library.interfaces.Collaborative;

import library.util.Debugger;

public aspect ExitSynchronizingAspect introduces ExitSynchronizing 
{	
	//declare parents: ExitSynchronizing implements Collaborative;

	private Object ExitSynchronizing.exitCondition = new Object();
	
	private int ExitSynchronizing.numberOfWaitingMembers = 0;
	private boolean ExitSynchronizing.synchronizationActive = true;
	
	public void ExitSynchronizing.setSynchronizationActive(boolean value)
	{
		synchronizationActive = value;
		
		if(value == false)
		{
			int totalParticipants = this.getParticipantCount();
			
			this.setNumberOfWaitingMembers(totalParticipants);

			synchronized(exitCondition)
			{
				exitCondition.notifyAll();
			}
		}
	}
	
	public boolean ExitSynchronizing.isSynchronizationActive()
	{
		return synchronizationActive;
	}
	
	public void ExitSynchronizing.setNumberOfWaitingMembers(int newValue)
	{
		numberOfWaitingMembers = newValue;
	}
	
	public int ExitSynchronizing.getNumberOfWaitingMembers()
	{
		return numberOfWaitingMembers;
	}
	
	public void ExitSynchronizing.incrementNumberOfWaitingMembers()
	{
		numberOfWaitingMembers++;
	}
	
	public Object ExitSynchronizing.getExitConditionObject()
	{
		return exitCondition;
	}
	
	pointcut memberLeavesTheContext(ExitSynchronizing synchExitContext):
		call(public void Collaborative.removeParticipant(..)) && target(synchExitContext);
	
	before(ExitSynchronizing synchExitContext) : memberLeavesTheContext(synchExitContext)
	{
		Debugger.printMessage("ExitSynchronizing",Debugger.CONTEXT_ASPECT,synchExitContext,"The call for removing the participant -" + Thread.currentThread().getName() + "-" + " has been intercepted.",2);
		
		if(synchExitContext.isSynchronizationActive())
		{
			synchronized(synchExitContext.getExitConditionObject())
			{
				synchExitContext.incrementNumberOfWaitingMembers();
			
				if(synchExitContext.getNumberOfWaitingMembers() == synchExitContext.getParticipantCount())
				{
					Debugger.printMessage("ExitSynchronizing",Debugger.CONTEXT_ASPECT,synchExitContext,"The synchronization process has been completed." + " The participants may now leave the context.",1);
					
					synchExitContext.getExitConditionObject().notifyAll();
				}
				else
				{
					
					Debugger.printMessage("ExitSynchronizing",Debugger.CONTEXT_ASPECT,synchExitContext,"The participant -" + Thread.currentThread().getName() + "-" + " is forced to wait at the exit of the context.",1);
					
					try
					{
						synchExitContext.getExitConditionObject().wait();
					}
					catch(InterruptedException ex){}
				}
			}
		}
		
		Debugger.printMessage("ExitSynchronizing",Debugger.CONTEXT_ASPECT,synchExitContext,"The call for removing the participant -" + Thread.currentThread().getName() + "-" + " has been released.",2);
	}
	
}
