package library.aspects.context;

import library.interfaces.Nested;
import library.interfaces.Context;
import library.interfaces.ContextParticipant;

import java.util.Set;
import java.util.HashSet;

import library.util.Debugger;

public aspect NestedAspect introduces Nested
{
	//declare parents: Nested implements Context;

	private Context Nested.parentContext = null;
	private Set<Context> Nested.childrenContext = null;
	
	public void Nested.setParentContext(Context parent)
	{
		parentContext = parent;
	}
	
	public Context Nested.getParentContext()
	{
		return parentContext;
	}
	
	public boolean Nested.hasAParent()
	{
		return (parentContext != null);
	}
	
	public void Nested.initializeChildrenContext()
	{
		childrenContext = new HashSet<Context>();
	}
	
	public void Nested.addChildContext(Context child)
	{
		childrenContext.add(child);
	}
	
	public void Nested.removeChildContext(Context child)
	{
		childrenContext.remove(child);
	}
	
	public Set<Context> Nested.getAllChildrenContext()
	{
		return childrenContext;
	}
	
	pointcut nestedConstructorExecution(Nested nestedContext):
    	target(nestedContext) && execution(public Nested+.new(..)) ;
	
	after(Nested nestedContext) : nestedConstructorExecution(nestedContext)
	{
		Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,nestedContext,"Constructor execution call (after) has been intercepted.",2);
		
		nestedContext.initializeChildrenContext();
		
		ContextParticipant participant = (ContextParticipant) Thread.currentThread();
		
		if(null != participant.getCurrentContext())
		{
			Nested parentContext = (Nested) participant.getCurrentContext();
			
			parentContext.addChildContext(nestedContext);
			
			Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,nestedContext,"The newly created context becomes the child.",1);
			
			nestedContext.setParentContext(parentContext);
		}
		Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,nestedContext,"Constructor execution call (after) has been released.",2);
	}
	
	pointcut memberRemoval(Nested context):
		call(public void dismissParticipant(..)) && target(context);
	
	Object around(Nested context) : memberRemoval(context)
	{
		Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,context,"The dismiss participant call has been intercepted.",2);
		
		Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,context,"The dismiss participant call is left to proceed.",2);
		
		Object returningObject = proceed(context);
		
		Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,context,"The dismiss participant call has been re-intercepted.",2);
		
		if(context.hasAParent())
		{
			ContextParticipant member = (ContextParticipant) thisJoinPoint.getArgs()[0];
			
			if(context.getParentContext().hasParticipant(member))
			{
				member.setCurrentContext(context.getParentContext());
				
				Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,context,"The participant -" + Thread.currentThread().getName() + "- "
						+ "is placed in the parent context -" + context.getParentContext().getName() + "-.",1);
			}
		}
		Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,context,"The dismiss participant call has been released.",2);
		
		return returningObject;
	}
	
	pointcut nestedContextDestruction(Nested nestedContext):
		target(nestedContext) && call(public void inactivateContext(..));
	
	before(Nested nestedContext) : nestedContextDestruction(nestedContext)
	{
		Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,nestedContext,"Inactivation call (before) has been intercepted.",2);
		
		if(nestedContext.hasAParent())
		{
			Nested parentContext = (Nested) nestedContext.getParentContext();
			
			Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,nestedContext,"The child context gets removed from the list of the parent.",1);
			
			parentContext.removeChildContext(nestedContext);
		}
		
		Debugger.printMessage("Nested",Debugger.CONTEXT_ASPECT,nestedContext,"Inactivation call (before) has been released.",2);
	}
}
