package library.aspects.context;

import library.interfaces.OutcomeAware;
import library.interfaces.OutcomeAffecting;
import library.interfaces.Context;

import library.util.Debugger;

public aspect OutcomeAwareAspect introduces OutcomeAware
{
	declare parents: OutcomeAware implements Context;
	
	private boolean OutcomeAware.contextOutcome = true; // positive outcome
	
	public boolean OutcomeAware.hasPositiveOutcome()
	{
		return contextOutcome;
	}
	
	public void OutcomeAware.setOutcome(boolean value)
	{
		contextOutcome = value;
		
		Debugger.printMessage("OutcomeAware",Debugger.CONTEXT_ASPECT,this,"The outcome has been set as: " + value + ".",1);
	}
	
	public void OutcomeAware.notifyParticipantAboutOutcome(OutcomeAffecting member)
	{		
		// if what member thinks is different than the outcome, give him the exception
		
		if( (member.didVotePositive() && !hasPositiveOutcome() ) || // if member said yes but context is aborted
				(!member.didVotePositive() && hasPositiveOutcome()) ) // if member said no but context is committed
		{
			Debugger.printMessage("OutcomeAware",Debugger.CONTEXT_ASPECT,this,"The context will need to notify the participant about the outcome of the context",1);
			
			member.getNotifiedByContext("The outcome of the context turns out to be different than what you think!");
			
			Debugger.printMessage("OutcomeAware",Debugger.CONTEXT_ASPECT,this,"The context has notified the participant about the outcome of the context",2);
		}
	}
	
	pointcut memberRemoval(OutcomeAware context):
		call(public void dismissParticipant(..)) && target(context);
	
	Object around(OutcomeAware context) : memberRemoval(context)
	{	
		Debugger.printMessage("OutcomeAware",Debugger.CONTEXT_ASPECT,context,"The dismiss participant call has been intercepted.",2);
		
		Debugger.printMessage("OutcomeAware",Debugger.CONTEXT_ASPECT,context,"The dismiss participant call is left to proceed.",2);
		
		Object returningObject = proceed(context);

		Debugger.printMessage("OutcomeAware",Debugger.CONTEXT_ASPECT,context,"The dismissParticipant call has been re-intercepted.",2);
		
		OutcomeAffecting participant = (OutcomeAffecting) thisJoinPoint.getArgs()[0];
		
		// Check to see if what he thought was correct
		context.notifyParticipantAboutOutcome(participant);
		
		Debugger.printMessage("OutcomeAware",Debugger.CONTEXT_ASPECT,context,"The dismissParticipant call has been released.",2);
		
		return returningObject;
	}
	
}
