package library.aspects.context;

import library.interfaces.OutcomeVoted;
import library.interfaces.Collaborative;
import library.interfaces.OutcomeAware;
import library.interfaces.OutcomeAffecting;
import library.interfaces.Terminatable;

import library.util.Debugger;

// Right now, just supports one type of voting scheme: All members have to agree!

public aspect OutcomeVotedAspect 
{
	//declare parents: OutcomeVoted implements /*Collaborative, OutcomeAware,*/ Terminatable;

	private int OutcomeVoted.numberOfPositiveVotes = 0;
	
	public void OutcomeVoted.handlePositiveVote(OutcomeAffecting member)
	{
		numberOfPositiveVotes++;
		
		if(numberOfPositiveVotes == this.getParticipantCount())
		{
			setOutcome(true);
		}	
	}
	
	public void OutcomeVoted.handleNegativeVote(OutcomeAffecting member)
	{	
		setOutcome(false);
		
		Debugger.printMessage("Terminatable",Debugger.CONTEXT_ASPECT,this,"The termination process is initiated.",1);
		
		this.terminate();
	}
	
	pointcut memberVotes(OutcomeVoted context, OutcomeAffecting member):
			call(public void OutcomeAware.setOutcome(..)) && target(context) && this(member);
	
	Object around(OutcomeVoted context, OutcomeAffecting member) : memberVotes(context,member)
	{
		Debugger.printMessage("OutcomeVoted",Debugger.CONTEXT_ASPECT,context,"The attempt to modify the outcome, by participant -" + Thread.currentThread().getName() + 
				"- has been intercepted.",2);
		
		// Get the vote of the participant
		boolean vote = (Boolean) thisJoinPoint.getArgs()[0];
		
		if(vote) // It is a positive vote
		{
			Debugger.printMessage("OutcomeVoted",Debugger.CONTEXT_ASPECT,context,"The positive vote of the participant -" + Thread.currentThread().getName() + 
					"- is being processed.",1);
			
			context.handlePositiveVote(member);
			
			Debugger.printMessage("OutcomeVoted",Debugger.CONTEXT_ASPECT,context,"The positive vote of the participant -" + Thread.currentThread().getName() + 
					"- has been processed.",2);
		}
		else // It is a negative vote
		{
			Debugger.printMessage("OutcomeVoted",Debugger.CONTEXT_ASPECT,context,"The negative vote of the participant -" + Thread.currentThread().getName() + 
					"- is being processed.",1);
			
			context.handleNegativeVote(member);
			
			Debugger.printMessage("OutcomeVoted",Debugger.CONTEXT_ASPECT,context,"The negative vote of the participant -" + Thread.currentThread().getName() + 
					"- has been processed.",2);
		}
		
		Debugger.printMessage("OutcomeVoted",Debugger.CONTEXT_ASPECT,context,"The attempt to modify the outcome, by participant -" + Thread.currentThread().getName() + 
				"- has been released.",2);
		
		return null;
	}
}
