package library.aspects.context;

import library.interfaces.Pausable;
import library.interfaces.Context;
import library.interfaces.ContextParticipant;

import library.util.Debugger;

public aspect PausableAspect introduces Pausable
{
	//declare parents: Pausable implements Context;
	
	private boolean Pausable.pausingActive = false;
	
	private int Pausable.numberOfMembersToPause = 0;
	
	private Object Pausable.pauseCondition = new Object();
	
	public void Pausable.pause()
	{	
		numberOfMembersToPause = this.numberOfParticipants();
		
		Debugger.printMessage("Pausable",Debugger.CONTEXT_ASPECT,this,"A pause request has been received. " + 
				numberOfMembersToPause + " participants will be paused.",1);
		
		pausingActive = true;
	}
		
	public void Pausable.resume()
	{
		pausingActive = false;
		
		numberOfMembersToPause = 0;
		
		synchronized(pauseCondition)
		{
			pauseCondition.notifyAll();
		}
	}
		
	public void Pausable.decrementNumberOfMembersToPause()
	{
		numberOfMembersToPause--;
	}
	
	public int Pausable.getNumberOfMembersToPause()
	{
		return numberOfMembersToPause;
	}
	
	public void Pausable.setNumberOfMembersToPause(int newValue)
	{
		numberOfMembersToPause = newValue;
	}
	
	public Object Pausable.getPauseCondition()
	{
		return pauseCondition;
	}
	
	public boolean Pausable.isInPausingMode()
	{
		return pausingActive;
	}
	
	public void Pausable.setPausingActive(boolean value)
	{
		pausingActive = value;
	}
		
	private static boolean isContextInPausingMode(Pausable context)
	{
		return context.isInPausingMode();
	}
	
	public void Pausable.pausingCompleted(){}

	public void Pausable.pauseTheParticipant(ContextParticipant participant, String methodName, Object[] args)
	{
		synchronized(getPauseCondition())
		{
			// If it is the addMember method, then it is a new member..
			// So, we should not modify the numberOfMembers variable
			if(!methodName.equals("assignParticipant"))
			{
				decrementNumberOfMembersToPause();
			}

			Debugger.printMessage("Pausable",Debugger.CONTEXT_ASPECT,this,"Participant [" + Thread.currentThread().getName() + "] " +
					"is being paused.",1);
			
			if(getNumberOfMembersToPause() == 0)
			{
				Debugger.printMessage("Pausable",Debugger.CONTEXT_ASPECT,this,"All the participants of the context have been paused",1);
				this.pausingCompleted();
			}
			try
			{
				getPauseCondition().wait();
			}
			
			catch(InterruptedException exception){}
		}
	}
	
	// Any communication with the context should be intercepted
	pointcut anyCommunicationWithContext(Pausable pausableContext) : 
		call(public * Context.*(..)) && target(pausableContext) && if(isContextInPausingMode(pausableContext)) 
			&& !call(public * Context.getName()) && !call(public * Context.setName(..)) && !within(*Aspect);

	before(Pausable pausableContext) : 
		anyCommunicationWithContext(pausableContext)
	{
		Debugger.printMessage("Pausable",Debugger.CONTEXT_ASPECT,pausableContext,"The activity (" + thisJoinPoint.getSignature().getName() + ") of the participant" +
				" [" + Thread.currentThread().getName() + "] " + "has been intercepted.",1);
		
		Object[] arguments = thisJoinPoint.getArgs();
		
		String methodName = (String) thisJoinPoint.getSignature().getName();
		
		ContextParticipant participant = (ContextParticipant) Thread.currentThread();
		
		pausableContext.pauseTheParticipant(participant, methodName, arguments);
		
		Debugger.printMessage("Pausable",Debugger.CONTEXT_ASPECT,pausableContext,"The activity (" + thisJoinPoint.getSignature().getName() + ") of the participant" +
				" [" + Thread.currentThread().getName() + "] " + "has been released.",2);
	}
	

}
