package library.aspects.context;

import library.interfaces.Pausable;
import library.interfaces.EntrySynchronizing;
import library.interfaces.PausableEntrySynchronizing;

import library.util.Debugger;

public aspect PausableEntrySynchronizingAspect 
{

	//declare parents: PausableEntrySynchronizing implements Pausable/*, EntrySynchronizing*/;

	pointcut pausingActivation(PausableEntrySynchronizing context):
		call(public void pause()) && target(context);
	
	after(PausableEntrySynchronizing context) : pausingActivation(context)
	{
		Debugger.printMessage("PausableEntrySynchronizing",Debugger.CONTEXT_ASPECT,context,"The participants, which are waiting at the entry of the context, will be taken into account during the pausing process.",1);
		
		// The synchronization process was complete, there is no one waiting at the entry
		if(context.getMinimumMemberCount() == 0)
		{
			// do nothing
		}
		
		// Decrement by the number of waiting participants
		else
		{
			int numberOfWaiters = context.getNumberOfWaitingParticipants();
			
			int numberToBePaused = context.getNumberOfMembersToPause();
			
			context.setNumberOfMembersToPause(numberToBePaused - numberOfWaiters);
		}	
		Debugger.printMessage("PausableEntrySynchronizing",Debugger.CONTEXT_ASPECT,context,"The participants, which are waiting at the entry of the context, have been taken into account during the pausing process.",2);
	}
}
