package library.aspects.context;

import library.interfaces.RecoveringDeferred;
import library.interfaces.Deferring;
import library.interfaces.Recovering;
import library.interfaces.VersionedContextAware;

import java.util.Set;

import library.util.Debugger;

public aspect RecoveringDeferredAspect 
{
	//declare parents : RecoveringDeferred implements /*Recovering,*/ Deferring;

	public void RecoveringDeferred.performDeferredRecovery()
	{
		Set<Object> modifiedObjects = getModifiedObjects(); // Assistance from Tracking
		
		if(modifiedObjects.isEmpty()) // If there is no modification, simply return
			return;
		
		for(Object obj: modifiedObjects)
		{	
			VersionedContextAware versioned = (VersionedContextAware) obj;
			
			// First determine the version that was created for this context
			int contextVersion = versioned.lookUpVersionForContext(this);
			
			// Delete that version since it is not necessary anymore
			versioned.deleteVersion(contextVersion);
			
			// Make sure that the context does not point to that version anymore
			versioned.unregisterContext(this);
		}

	}

	pointcut blockDeferring(RecoveringDeferred context) : 
		target(context) && call(public void Deferring.finalizeVersions());
	
	Object around(RecoveringDeferred context) : blockDeferring(context)
	{
		Debugger.printMessage("RecoveringDeferred",Debugger.CONTEXT_ASPECT,context,"The finalization of versions call has been intercepted.",2);
		
		if(!context.hasPositiveOutcome())
		{
			Debugger.printMessage("RecoveringDeferred",Debugger.CONTEXT_ASPECT,context,"The context will be performing a recovery deferred.",1);
			
			context.performDeferredRecovery();
			
			Debugger.printMessage("RecoveringDeferred",Debugger.CONTEXT_ASPECT,context,"The context has performed a recovery deferred.",2);
			
			Debugger.printMessage("RecoveringDeferred",Debugger.CONTEXT_ASPECT,context,"Recovery won't let Deferring finalize the local versions of the context.",1);
			
			return null;
		}
		else
		{
			Debugger.printMessage("RecoveringDeferred",Debugger.CONTEXT_ASPECT,context,"The finalization of versions call is left to proceed.",2);
			
			return proceed(context);
		}		
		
	}
}
