package library.aspects.context;

import library.interfaces.RecoveringInPlace;
import library.interfaces.Checkpointing;
import library.interfaces.Checkpointable;
import library.interfaces.Recovering;

import java.util.Set;

import library.util.Debugger;

public aspect RecoveringInPlaceAspect 
{
	declare parents : RecoveringInPlace implements /*Recovering,*/ Checkpointing;

	public void RecoveringInPlace.performInPlaceRecovery()
	{
		Set<Object> modifiedObjects = getModifiedObjects(); // Assistance from Tracking
		
		if(modifiedObjects.isEmpty()) // If there is no modification, simply return
			return;
		
		for(Object object : modifiedObjects)
		{
			Checkpointable checkpointable = (Checkpointable) object;
			
			checkpointable.restoreCheckPoint();
		}

	}


	pointcut recoveryTime(RecoveringInPlace context) : 
		target(context) && call(public void Checkpointing.finalizeCheckpoints());
	
	Object around(RecoveringInPlace context) : recoveryTime(context)
	{
		Debugger.printMessage("RecoveringInPlace",Debugger.CONTEXT_ASPECT,context,"The finalization of checkpoints call has been intercepted.",2);
		
		if(!context.hasPositiveOutcome())
		{
			Debugger.printMessage("RecoveringInPlace",Debugger.CONTEXT_ASPECT,context,"The context will be performing a recovery in place.",1);
			
			context.performInPlaceRecovery();
			
			Debugger.printMessage("RecoveringInPlace",Debugger.CONTEXT_ASPECT,context,"The context has performed a recovery in place.",2);
			
			Debugger.printMessage("RecoveringInPlace",Debugger.CONTEXT_ASPECT,context,"Recovery won't let Checkpointing finalize the checkpoints of the context.",1);
			
			return null;
		}
		else
		{
			Debugger.printMessage("RecoveringInPlace",Debugger.CONTEXT_ASPECT,context,"The finalization of checkpoints call is left to proceed.",2);
			
			return proceed(context);
		}		
		
	}
}
