package library.aspects.context;

import library.interfaces.ContextParticipant;
import library.interfaces.Terminatable;
import library.util.Debugger;

public aspect TerminatableAspect introduces Terminatable
{
	//declare parents: Terminatable extends Pausable;

	private boolean Terminatable.terminationActive = false;

	public void Terminatable.terminate()
	{
		this.pause();
		
		terminationActive = true;
	}
	
	public boolean Terminatable.isInTerminationMode()
	{
		return terminationActive;
	}
	
	public void Terminatable.setTerminationActive(boolean value)
	{
		terminationActive = value;
	}
	
	private static boolean isContextInTerminationMode(Terminatable context)
	{
		return context.isInTerminationMode();
	}
	
	pointcut memberCommunication(Terminatable context):
		call(public * pauseTheParticipant(..)) && target(context) && if(isContextInTerminationMode(context));
	
	Object around(Terminatable context): memberCommunication(context)
	{
		Debugger.printMessage("Terminatable",Debugger.CONTEXT_ASPECT,context,"The call to pause the participant has been intercepted",2);
		
		ContextParticipant participant = (ContextParticipant) thisJoinPoint.getArgs()[0];
	
		String methodName = (String) thisJoinPoint.getArgs()[1];
		
		if(methodName.equals("dismissParticipant"))
		{
			// The participant is already leaving.. No need to take any action
		}
		else
		{
		
			Debugger.printMessage("Terminatable",Debugger.CONTEXT_ASPECT,context,"The participant will be removed from the context.",1);
			
			context.dismissParticipant(participant);
			
			participant.receiveTerminationSignal("Context has been terminated");
			
			Debugger.printMessage("Terminatable",Debugger.CONTEXT_ASPECT,context,"The participant has been removed from the context.",1);
		}
		
		Debugger.printMessage("Terminatable",Debugger.CONTEXT_ASPECT,context,"The call to pause the participant has been released",2);
		
		return null;
	}
}
