package library.aspects.context;

import library.interfaces.Nested;
import library.interfaces.Tracing;
import library.interfaces.TracingNested;

import library.util.Debugger;

import library.util.ObjectTrace;

public aspect TracingNestedAspect
{
	//declare parents: TracingNested implements /*Nested, */Tracing;
	
	private boolean TracingNested.mergeTracesOnExit = true;
		
	public void TracingNested.setMergeTracesOnExit(boolean value)
	{
		mergeTracesOnExit = value;
	}
		
	public boolean TracingNested.getMergeTracesOnExit()
	{
		return mergeTracesOnExit;
	}
	
	public void TracingNested.finalizeNestedTraces()
	{
		if(this.getMergeTracesOnExit())
		{	

			this.transferTracesToParent();

		}	
	}
	
	public void TracingNested.transferTracesToParent()
	{
		TracingNested parentContext = (TracingNested) this.getParentContext();
		
		for(ObjectTrace trace: this.getAllTraces())
		{
			parentContext.addObjectTrace(trace);
		}		
	}
	
	pointcut trackFinalization(TracingNested context):
		call(public * finalizeTraces(..)) && target(context);
	
	Object around(TracingNested context) : trackFinalization(context)
	{	
		Debugger.printMessage("TracingNested",Debugger.CONTEXT_ASPECT,context,"The trace finalization call has been intercepted.",2);
		
		if(!context.hasAParent())
		{
			Debugger.printMessage("TracingNested",Debugger.CONTEXT_ASPECT,context,"The trace finalization call has been released.",2);
			
			return proceed(context);
		}
		else
		{
			Debugger.printMessage("TracingNested",Debugger.CONTEXT_ASPECT,context,"The finalization will be carried out along with nesting rules.",1);
			
			context.finalizeNestedTraces();
			
			Debugger.printMessage("TracingNested",Debugger.CONTEXT_ASPECT,context,"The trace finalization call has been released.",2);
			
			return null;
		}
	}
}
