package library.aspects.context;

import library.interfaces.TwoPhaseLocking;
import library.interfaces.TwoPhaseLockingNested;
import library.interfaces.Nested;
import library.interfaces.Lockable;
import library.interfaces.Traceable;
import library.util.AccessTypes;

import library.util.Debugger;

import java.util.Set;
import java.util.LinkedHashSet;

public aspect TwoPhaseLockingNestedAspect introduces TwoPhaseLockingNested
{
	//declare parents: TwoPhaseLockingNested implements TwoPhaseLocking/*, Nested*/;

	public Set<String> TwoPhaseLockingNested.lentReadLocks = new LinkedHashSet<String>();
	public Set<String> TwoPhaseLockingNested.lentWriteLocks = new LinkedHashSet<String>();
	
	public Set<String> TwoPhaseLockingNested.borrowedReadLocks = new LinkedHashSet<String>();
	public Set<String> TwoPhaseLockingNested.borrowedWriteLocks = new LinkedHashSet<String>();
	
	private Object TwoPhaseLockingNested.lendingCondition = new Object();
	
	public Set<String> TwoPhaseLockingNested.getLentReadLocks()
	{
		return lentReadLocks;
	}
	
	public Set<String> TwoPhaseLockingNested.getLentWriteLocks()
	{
		return lentWriteLocks;
	}
	
	public Set<String> TwoPhaseLockingNested.getBorrowedReadLocks()
	{
		return borrowedReadLocks;
	}
	
	public Set<String> TwoPhaseLockingNested.getBorrowedWriteLocks()
	{
		return borrowedWriteLocks;
	}
	
	public void TwoPhaseLockingNested.lendReadLockOfObjectToContext(String objectName, TwoPhaseLockingNested childContext)
	{
		Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The READ transactional lock of object [" + objectName + "]"
				+ " has been lent to context [" + childContext.getName() + "]",1);
		
		this.lentReadLocks.add(objectName);
		
		childContext.getBorrowedReadLocks().add(objectName);
	}
	
	public void TwoPhaseLockingNested.lendWriteLockOfObjectToContext(String objectName, TwoPhaseLockingNested childContext)
	{
		Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The WRITE transactional lock of object [" + objectName + "]"
				+ " has been lent to context [" + childContext.getName() + "]",1);
		
		this.lentWriteLocks.add(objectName);
		
		childContext.getBorrowedWriteLocks().add(objectName);

	}
	
	public boolean TwoPhaseLockingNested.didLendReadLockOfObject(String objectName)
	{
		return lentReadLocks.contains(objectName);
	}
	
	public boolean TwoPhaseLockingNested.didLendWriteLockOfObject(String objectName)
	{
		return lentWriteLocks.contains(objectName);
	}
	
	public boolean TwoPhaseLockingNested.doesPossessTheReadLockOfObject(Lockable object)
	{
		String objectName = ((Traceable)object).getMyName();
		
		return hasAcquiredReadLockOfObject(object) && !didLendReadLockOfObject(objectName);
	}
	
	public boolean TwoPhaseLockingNested.doesPossessTheWriteLockOfObject(Lockable object)
	{
		String objectName = ((Traceable)object).getMyName();
		
		return hasAcquiredWriteLockOfObject(object) && !didLendWriteLockOfObject(objectName);
	}
	
	public void TwoPhaseLockingNested.waitForLentReadLockOfObject(Lockable object, TwoPhaseLockingNested context)
	{
		Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The child context [" + context.getName() + "] is waiting to borrow" +
				"the READ lock of object [" + ((Traceable)object).getMyName() + "] from this context.",1);
		
		synchronized(lendingCondition)
		{
			while(!doesPossessTheReadLockOfObject(object))
			{
				try
				{
					lendingCondition.wait();
				}
				catch(InterruptedException ex){}
			}
			
			String objectName = ((Traceable)object).getMyName();
			
			lendReadLockOfObjectToContext(objectName,context);
			
		}
	}
	
	public void TwoPhaseLockingNested.waitForLentWriteLockOfObject(Lockable object, TwoPhaseLockingNested context)
	{
		Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The child context [" + context.getName() + "] is waiting to borrow" +
				"the WRITE lock of object [" + ((Traceable)object).getMyName() + "] from this context.",1);
		
		synchronized(lendingCondition)
		{
			while(!doesPossessTheWriteLockOfObject(object))
			{
				try
				{
					lendingCondition.wait();
				}
				catch(InterruptedException ex){}
			}
			
			String objectName = ((Traceable)object).getMyName();
			
			lendWriteLockOfObjectToContext(objectName,context);
		}
	}	
	
	public void TwoPhaseLockingNested.releaseAllNestedLocks()
	{
		TwoPhaseLockingNested parentContext = null;
		
		for(String objectName : this.getBorrowedReadLocks())
		{
			parentContext = (TwoPhaseLockingNested) this.getParentContext();
			
			while(parentContext != null)
			{
				if(parentContext.getLentReadLocks().contains(objectName))
				{
					Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The borrowed READ lock of object [" 
							+ objectName + "] has been given back to the parent context [" + parentContext.getName() + "]",1);
					
					parentContext.getLentReadLocks().remove(objectName);
					
					this.getBorrowedReadLocks().remove(objectName);
					
					break;
				}
				
				parentContext = (TwoPhaseLockingNested) parentContext.getParentContext();
		
			}
		}
		
		
		for(String objectName : this.getBorrowedWriteLocks())
		{
			parentContext = (TwoPhaseLockingNested) this.getParentContext();
		
			while(parentContext != null)
			{
				if(parentContext.getLentWriteLocks().contains(objectName))
				{
					Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The borrowed WRITE lock of object [" 
							+ objectName + "] has been given back to the parent context [" + parentContext.getName() + "]",1);
					
					parentContext.getLentWriteLocks().remove(objectName);
					
					this.getBorrowedWriteLocks().remove(objectName);
					
					break;
				}
				
				parentContext = (TwoPhaseLockingNested) parentContext.getParentContext();
			}
		}
		
		parentContext = (TwoPhaseLockingNested) this.getParentContext();
		
		synchronized(parentContext.lendingCondition)
		{
			parentContext.lendingCondition.notifyAll();
		}
		
	}
	
	pointcut regularReleasingStep(TwoPhaseLockingNested currentContext) : 
		call(public * releaseAllAcquiredLocks(..)) && target(currentContext);
	
	Object around(TwoPhaseLockingNested currentContext): regularReleasingStep(currentContext)
	{
		if(!currentContext.hasAParent())
		{
			return proceed(currentContext);
		}
		
		else
		{
			currentContext.releaseAllNestedLocks();
		}
	
		return null;
	}
	
	public void TwoPhaseLockingNested.acquireNecessaryNestedLock(Lockable lockableObject, String methodName)
	{
		String accessType = ((Traceable)lockableObject).getAccessTypeOfMethod(methodName);
		String objectName = ((Traceable)lockableObject).getMyName();
		
		if(accessType.equals(AccessTypes.READ))
		{
			if(doesPossessTheReadLockOfObject(lockableObject) || doesPossessTheReadLockOfObject(lockableObject))
			{
				Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The lock is already owned by the context.",1);
			}
			else if(getBorrowedReadLocks().contains(objectName) || getBorrowedWriteLocks().contains(objectName))
			{
				Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The lock is already borrowed by the context.",1);
			}
			else
			{
				TwoPhaseLockingNested parentContext = (TwoPhaseLockingNested) getParentContext();
				boolean lockAcquired = false;
				
				while(parentContext != null)
				{
					if(parentContext.doesPossessTheReadLockOfObject(lockableObject))
					{
						parentContext.lendReadLockOfObjectToContext(objectName,this);
						
						lockAcquired = true;
						break;
					}
					
					else if(parentContext.doesPossessTheWriteLockOfObject(lockableObject))
					{
						parentContext.lendWriteLockOfObjectToContext(objectName,this);
						
						lockAcquired = true;
						break;
					}
					
					else if(parentContext.didLendReadLockOfObject(objectName))
					{
						parentContext.waitForLentReadLockOfObject(lockableObject,this);
						
						lockAcquired = true;
						break;
					}
					
					else if(parentContext.didLendWriteLockOfObject(objectName))
					{
						parentContext.waitForLentWriteLockOfObject(lockableObject,this);
						
						lockAcquired = true;
						break;
					}
					
					parentContext = (TwoPhaseLockingNested) parentContext.getParentContext();
				}
				
				if(!lockAcquired)
				{
					Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The READ lock of an object is about to be acquired.",1);
					
					lockableObject.acquireReadLock("2PL");
					
					Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The READ lock of an object has been acquired.",2);
				}
			}
				
		}
		
		else if(accessType.equals(AccessTypes.WRITE) || accessType.equals(AccessTypes.UPDATE))
		{
			if(doesPossessTheWriteLockOfObject(lockableObject))
			{
				// just use it, no problems
			}
			
			else if(doesPossessTheReadLockOfObject(lockableObject))
			{
				upgradeTheAcquiredReadLockOfObject(lockableObject);
			}
			
			else if(getBorrowedWriteLocks().contains(objectName))
			{
				Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The lock is already borrowed by the context.",1);
			}
			
			else
			{
				TwoPhaseLockingNested parentContext = (TwoPhaseLockingNested) getParentContext();
				boolean lockAcquired = false;
				
				while(parentContext != null)
				{
					if(parentContext.doesPossessTheReadLockOfObject(lockableObject))
					{
						parentContext.upgradeTheAcquiredReadLockOfObject(lockableObject);
						
						parentContext.lendWriteLockOfObjectToContext(objectName,this);
						
						lockAcquired = true;
						break;
					}
					
					else if(parentContext.doesPossessTheWriteLockOfObject(lockableObject))
					{
						parentContext.lendWriteLockOfObjectToContext(objectName,this);
						
						lockAcquired = true;
						break;
					}
					
					
					else if(parentContext.didLendReadLockOfObject(objectName))
					{
						parentContext.waitForLentReadLockOfObject(lockableObject,this);
						
						parentContext.upgradeTheAcquiredReadLockOfObject(lockableObject);
						
						parentContext.lendWriteLockOfObjectToContext(objectName,this);
						
						lockAcquired = true;
						break;
					}
					
					
					else if(parentContext.didLendWriteLockOfObject(objectName))
					{
						parentContext.waitForLentWriteLockOfObject(lockableObject,this);
						
						lockAcquired = true;
						break;
					}
					
					parentContext = (TwoPhaseLockingNested) parentContext.getParentContext();
				}
				
				if(!lockAcquired)
				{
					Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The WRITE lock of an object is about to be acquired.",1);
					
					lockableObject.acquireWriteLock("2PL");
					
					Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,this,"The WRITE lock of an object has been acquired.",2);
				}
			}
		}
	}
	
	pointcut memberPerformingWork(TwoPhaseLockingNested currentContext) : 
		call(public * acquireNecessaryLock(..)) && target(currentContext);
	
	Object around(TwoPhaseLockingNested currentContext): memberPerformingWork(currentContext)
	{	
		Object returningObject = null;
		
		Object[] arguments = thisJoinPoint.getArgs();
		
		Object accessedObject = arguments[0];
		String methodName = (String) arguments[1];
		
		Lockable lockableObject = (Lockable) accessedObject;
		
		if(!currentContext.hasAParent())
		{
			return proceed(currentContext);
		}
		
		else
		{
			Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,currentContext,"The appropriate nested 2PL lock is being acquired",2);
			
			currentContext.acquireNecessaryNestedLock(lockableObject,methodName);
		}
		
		Debugger.printMessage("TwoPhaseLockingNested",Debugger.CONTEXT_ASPECT,currentContext,"The appropriate nested 2PL lock has been acquired",2);
		
		return returningObject;
	}
}
