package library.aspects.context;

import library.interfaces.TwoPhaseLockingNested;
import library.interfaces.Recovering;
import library.interfaces.TwoPhaseLockingNestedRecovering;
import library.interfaces.Lockable;
import library.interfaces.Traceable;

import library.util.Debugger;

import java.util.Set;

public aspect TwoPhaseLockingNestedRecoveringAspect
{
	//declare parents: TwoPhaseLockingNestedRecovering implements TwoPhaseLockingNested/*, Recovering*/;

	public void TwoPhaseLockingNestedRecovering.throwAwayNotBorrowedLocks()
	{
		// Retrieve the list of all read objects
		for(String readObjectName : getReadObjectNames())
		{
			// Filter the ones that haven't been borrowed
			if(!this.getBorrowedReadLocks().contains(readObjectName))
			{
				Set<Object> readObjects = getReadObjects();
				
				for(Object object : readObjects)
				{
					String objectName = ((Traceable)object).getMyName();
					
					if(objectName.equals(readObjectName))
					{
						Lockable lockable = (Lockable) object;
						
						Debugger.printMessage("TwoPhaseLockingNestedRecovering",Debugger.CONTEXT_ASPECT,this,"The READ lock of an object is being released.",1);
				
						lockable.releaseReadLock("2PL");
				
						Debugger.printMessage("TwoPhaseLockingNestedRecovering",Debugger.CONTEXT_ASPECT,this,"The READ lock of an object has been released.",2);
					}	
				}
			}
		}
		
		// Retrieve the list of all modified objects
		Set<String> allModifiedObjectNames = getWrittenObjectNames();
		allModifiedObjectNames.addAll(getUpdatedObjectNames());
		
		for(String modifiedObjectName : allModifiedObjectNames)
		{
			// Filter the ones that haven't been borrowed
			if(!this.getBorrowedWriteLocks().contains(modifiedObjectName))
			{
				Set<Object> modifiedObjects = getModifiedObjects();
				
				for(Object object : modifiedObjects)
				{
					String objectName = ((Traceable)object).getMyName();
					
					if(objectName.equals(modifiedObjectName))
					{
						Lockable lockable = (Lockable) object;
						
						Debugger.printMessage("TwoPhaseLockingNestedRecovering",Debugger.CONTEXT_ASPECT,this,"The WRITE lock of an object is being released.",1);
				
						lockable.releaseWriteLock("2PL");
				
						Debugger.printMessage("TwoPhaseLockingNestedRecovering",Debugger.CONTEXT_ASPECT,this,"The WRITE lock of an object has been released.",2);

					}	
				}
			}
		}
	
	}
	
	pointcut contextDestruction(TwoPhaseLockingNestedRecovering currentContext) : 
		call(public * releaseAllNestedLocks(..)) && target(currentContext);
	
	Object around(TwoPhaseLockingNestedRecovering currentContext): contextDestruction(currentContext)
	{
		
		if(!currentContext.hasPositiveOutcome())
		{
			Debugger.printMessage("TwoPhaseLockingNestedRecovering",Debugger.CONTEXT_ASPECT,currentContext,"The new locks that this context has acquired will be released along with recovering principles..",1);
			
			currentContext.throwAwayNotBorrowedLocks();
			
			Debugger.printMessage("TwoPhaseLockingNestedRecovering",Debugger.CONTEXT_ASPECT,currentContext,"The new locks that this context has acquired were released along with recovering principles..",2);
		}
		
		return proceed(currentContext);
	
	}
}
