package library.aspects.object;

import library.interfaces.ContextAware;
import library.interfaces.ContextParticipant;
import library.interfaces.Context;
import library.interfaces.Named;

import library.util.Debugger;

public aspect ContextAwareAspect 
{
	declare parents: ContextAware implements Named;
	
	public void ContextAware.notifyContextAboutInvocation(Context targetContext, ContextParticipant participant, String methodName, Object args[])
	{
		targetContext.participantPerformsWork(participant, this, methodName, args);
	}

	pointcut memberAccessesContextAwareObject(ContextAware object, ContextParticipant participant):
    	target(object) && this(participant) && call(public * ContextAware+.*(..));
	
	before(ContextAware object,ContextParticipant participant) : memberAccessesContextAwareObject(object,participant)
	{
		Debugger.printMessage("ContextAware",Debugger.OBJECT_ASPECT,object,"The method call (before) to the object has been intercepted.",2);
		
		Context targetContext = participant.getCurrentContext();
		
		// If the member is not currently in a context, the call should be treated as a regular call
		if(null == targetContext)
		{
			// Do not do anything... Versioned will forward the call to the main version
		}
		
		else // Notify the context about the invocation
		{
			Object args[] = thisJoinPoint.getArgs();
			String methodName = thisJoinPoint.getSignature().getName();
			
			Debugger.printMessage("ContextAware",Debugger.OBJECT_ASPECT,object,"The object will now " +
					"notify the context [" + targetContext.getName() + "].",1);
			
			object.notifyContextAboutInvocation(targetContext,participant,methodName,args);
			
			Debugger.printMessage("ContextAware",Debugger.OBJECT_ASPECT,object,"The object has done with " +
					"notifying the context [" + targetContext.getName() + "].",2);
		}
		
		Debugger.printMessage("ContextAware",Debugger.OBJECT_ASPECT,object,"The method call (before) to the object has been released.\n",2);
	
	}
	
}
