package library.aspects.object;

import java.util.Set;
import java.util.LinkedHashSet;

import library.interfaces.Context;
import library.interfaces.ContextTracing;
import library.interfaces.ContextAware;
import library.interfaces.AccessClassified;

import library.context.ContextManager;

import library.util.ContextTrace;

import library.util.Debugger;

public aspect ContextTracingAspect introduces ContextTracing 
{
	declare parents: ContextTracing implements ContextAware/*, AccessClassified*/;

	public Set<ContextTrace> ContextTracing.contexts = new LinkedHashSet<ContextTrace>();
	
	public Set<Context> ContextTracing.getAllContexts()
	{
		Debugger.printMessage("ContextTracing",Debugger.OBJECT_ASPECT,this,"The set of all traced contexts is being returned.",1);
		
		Set<Context> allContexts = new LinkedHashSet<Context>();
		
		for(ContextTrace track: contexts)
		{
			allContexts.add(ContextManager.getContextByName(track.getTrackedContext()));
		}
		
		return allContexts;
	}
	
	public void ContextTracing.setContextList(Set<ContextTrace> newSet)
	{
		contexts = newSet;
	}
	
	public void ContextTracing.addNewContext(Context newContext, String type)
	{		
		ContextTrace newTrack = new ContextTrace(newContext.getName(),type);
		
		if(contexts.add(newTrack))
		{
			Debugger.printMessage("ContextTracing",Debugger.OBJECT_ASPECT,this,"The object has tracked a new context: " + newContext.getName() + ".",1);

		}
	}

	pointcut memberAccessesContextTracingObject(ContextTracing object):
    	target(object) && call(public void notifyContextAboutInvocation(..));
	
	before(ContextTracing object) : memberAccessesContextTracingObject(object)
	{
		Debugger.printMessage("ContextTracing",Debugger.OBJECT_ASPECT,object,"The notification call to the context has been intercepted.",2);
		
		Object arguments[] = thisJoinPoint.getArgs();
				
		Context trackedContext = (Context) arguments[0];
		String methodName = (String) arguments[2];
		
		String accessType = object.getAccessTypeOfMethod(methodName);
		
		object.addNewContext(trackedContext,accessType);
		
		Debugger.printMessage("ContextTracing",Debugger.OBJECT_ASPECT,object,"The notification call to the context has been returned.",2);
	}
}
