package library.aspects.object;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import java.lang.reflect.*;
import library.interfaces.Copyable;

import library.util.Debugger;

public aspect CopyableAspect introduces Copyable 
{
	declare parents: Copyable implements Serializable;
		
	public Object Copyable.cloneState()
	{
		Debugger.printMessage("Copyable",Debugger.OBJECT_ASPECT,this,"The cloning of the object's state will now take place.",1);
		
		Object deepCopyOfOriginalObject = null;
		
		ByteArrayOutputStream memoryOutputStream = new ByteArrayOutputStream();
		ObjectOutputStream serializer;
		
		ByteArrayInputStream memoryInputStream;
		ObjectInputStream deserializer;
		
		try
		{
			serializer = new ObjectOutputStream(memoryOutputStream);
			serializer.writeObject(this);
			serializer.flush();
	    
			memoryInputStream = new ByteArrayInputStream(memoryOutputStream.toByteArray());
			deserializer = new ObjectInputStream(memoryInputStream);
			deepCopyOfOriginalObject = deserializer.readObject();
		}
		
		catch (IOException e)
		{
			e.printStackTrace();
        }
		
		catch (ClassNotFoundException e)
		{
			e.printStackTrace();
        }
		
		
		//Before returning the clone, a little hack...
		preventDuplicationOfInjectedFields(this,deepCopyOfOriginalObject);
		
		Debugger.printMessage("Copyable",Debugger.OBJECT_ASPECT,this,"The cloning operation is now complete.",2);
		
		return deepCopyOfOriginalObject;
	}   

	
	// The state of the this object gets replaced by the source parameter's state
	// In this case, the destination is the "this" object.. Therefore, the parameter becomes the source
	public void Copyable.replaceState(Object source)
	{
		Debugger.printMessage("Copyable",Debugger.OBJECT_ASPECT,this,"The object's state will now be replaced with another state.",1);
		
		copyFields(this,source);
		
		Debugger.printMessage("Copyable",Debugger.OBJECT_ASPECT,this,"The state replace operation is now complete.",2);
	}
	
	
	private static void preventDuplicationOfInjectedFields(Object thisObject, Object clonedState)
	{
		Class currentClass = clonedState.getClass();
		
		// Recursively copy the values from "source" to "this";
		// including all variables inherited from superclasses,
		// interfaces, inter-type declaration and composed objects
		while(currentClass != null)
		{
			// First get all the fields of the class
			Field[] allFields  = currentClass.getDeclaredFields();
	
	        // make the protected and private fields accessible
	        AccessibleObject.setAccessible(allFields, true);
	
	        // perform field by field copy of non-final variables
	        for (int i = 0; i < allFields.length; i++) 
	        {
	        	//System.out.println("Field name: " + allFields[i].getName());
	        	
	        	// The hack is here:
	        	// If it is an injected field, do not clone it..
	        	// Instead, it will refer back to the original one..
	        	if( !Modifier.isFinal(allFields[i].getModifiers()) 
	        			&& allFields[i].getName().startsWith("ajc"))
	        	{
	        		try 
	        		{
	        			allFields[i].set(clonedState, allFields[i].get(thisObject));
	        			//System.out.println("This variable was not cloned: " + allFields[i].getName());
	        		}
	        	 	catch (IllegalAccessException e) 
	        	 	{		
	        	 		e.printStackTrace();
	        	 	}
	        	}
	        }
	        
	        // Recursively copy the values of all super classes
	        currentClass = currentClass.getSuperclass();
		
		}
	}
	private static void copyFields(Object destination, Object source)
	{	 
		Class currentClass = source.getClass();
		
		// Recursively copy the values from "source" to "this";
		// including all variables inherited from superclasses,
		// interfaces, inter-type declaration and composed objects
		while(currentClass != null)
		{
			// First get all the fields of the class
			Field[] allFields  = currentClass.getDeclaredFields();
	
	        // make the protected and private fields accessible
	        AccessibleObject.setAccessible(allFields, true);
	
	        // perform field by field copy of non-final variables
	        for (int i = 0; i < allFields.length; i++) 
	        {
	        	//System.out.println("Field name: " + allFields[i].getName());
	        	
	        	// A little hack is inserted here:
	        	// Only copy the real state, not the state introduced by the aspects
	        	if( !Modifier.isFinal(allFields[i].getModifiers()) 
	        			&& !allFields[i].getName().startsWith("ajc"))
	        	{
	        		try 
	        		{
	        			allFields[i].set(destination, allFields[i].get(source));
	        			//System.out.println("Replaced variable name: " + allFields[i].getName());
	        		}
	        	 	catch (IllegalAccessException e) 
	        	 	{		
	        	 		e.printStackTrace();
	        	 	}
	        	}
	        }
	        
	        // Recursively copy the values of all super classes
	        currentClass = currentClass.getSuperclass();
		
		}
	}
}
