package library.aspects.object;

import library.interfaces.Copyable;
import library.interfaces.Lockable;
import library.interfaces.CopyableLockable;

import java.util.Map;
import java.util.Set;

import library.util.Debugger;
import library.util.Lock;

public aspect CopyableLockableAspect 
{
	declare parents: CopyableLockable implements Copyable/*, Lockable*/;
		
	pointcut stateCloning(CopyableLockable object) : 
		call(public Object cloneState(..)) && target(object);

	Object around(CopyableLockable object) : stateCloning(object)
	{	
		Object theClone = proceed(object);
		
		Debugger.printMessage("CopyableLockable",Debugger.OBJECT_ASPECT,object,"New locks will be associated for the new cloned state.",1);

		CopyableLockable clonedObject = (CopyableLockable) theClone;
		Map<String,Lock> locks = clonedObject.getLockCollection();
	
		Set<String> keys = locks.keySet();

		// For each lock belonging to a key, replace the lock with a new one
		for(String key: keys)
		{
			locks.put(key,new Lock());
		}

		Debugger.printMessage("CopyableLockable",Debugger.OBJECT_ASPECT,object,"New locks have been associated for the new cloned state.",2);

		return theClone;
		
	}
}

