package library.aspects.object;

import library.interfaces.Copyable;
import library.interfaces.Shared;
import library.interfaces.CopyableShared;

import library.util.Debugger;

public aspect CopyableSharedAspect 
{
	declare parents: CopyableShared implements Copyable/*, Shared*/;
	
	pointcut stateOperations(CopyableShared object) : 
		( call(public Object cloneState(..)) || call(public void replaceState(..)) )  
							&& target(object);

	Object around(CopyableShared object) : stateOperations(object)
	{
		Debugger.printMessage("CopyableShared",Debugger.OBJECT_ASPECT,object,"It will be made sure that state modifying operations are performed while nobody is else is working on the state.",1);
		
		Object returningObject = null;
		
		synchronized(object)
		{
			while(object.isCurrentlyInUse())
			{
				try
				{
					Debugger.printMessage("CopyableShared",Debugger.OBJECT_ASPECT,object,"A thread has to wait because the object is currently in use.",1);
					
					object.wait();
				}
				catch(InterruptedException exception){}
			}
		}
		
		returningObject = proceed(object);
		
		synchronized(object)
		{
			object.notifyAll();
		}
		
		Debugger.printMessage("CopyableShared",Debugger.OBJECT_ASPECT,object,"It was made sure that state modifying operations are performed while nobody is else is working on the state.",2);
		
		return returningObject;
	}
	
	
}


