package library.aspects.object;

import library.util.Lock;
import library.interfaces.Lockable;
import java.util.*;

import library.util.Debugger;

public aspect LockableAspect introduces Lockable
{
	private Map<String,Lock> Lockable.lockCollection = new HashMap<String,Lock>();
	
	public void Lockable.acquireReadLock(String owner)
	{
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The process of acquiring the READ lock for " + owner + " has started.",2);
		
		// If the owner already has an associated lock
		if(lockCollection.containsKey(owner))
		{
			Lock associatedLock = lockCollection.get(owner);
			
			associatedLock.getReadLock(owner,"");
			
			Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The READ lock of the object has been acquired for " + owner + ".",1);
		}
		
		//Create a new lock and associate that lock with the owner
		else
		{
			Lock newLock = new Lock();
			lockCollection.put(owner,newLock);
			
			Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"A new lock is created for " + owner + ".",1);
			
			newLock.getReadLock(owner,"");
			
			Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The READ lock of the object has been acquired for " + owner + ".",1);
		}
		
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The process of acquiring the READ lock for " + owner + " has ended.",2);
	}
	
	public void Lockable.acquireWriteLock(String owner)
	{
		
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The process of acquiring the WRITE lock for " + owner + " has started.",2);
		
		// If the owner already has an associated lock
		if(lockCollection.containsKey(owner))
		{
			Lock associatedLock = lockCollection.get(owner);
			
			associatedLock.getWriteLock(owner,"");
			
			Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The WRITE lock of the object has been acquired for " + owner + ".",1);
		}
		
		//Create a new lock and associate that lock with the owner
		else
		{
			Lock newLock = new Lock();
			lockCollection.put(owner,newLock);
			
			Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"A new lock is created for " + owner + ".",1);
			
			newLock.getWriteLock(owner,"");
			
			Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The WRITE lock of the object has been acquired for " + owner + ".",1);
		}
		
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The process of acquiring the WRITE lock for " + owner + " has ended.",2);
	}
	
	public void Lockable.releaseReadLock(String owner)
	{
		
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The process of releasing the READ lock for " + owner + " has started.",2);
		
		Lock associatedLock = lockCollection.get(owner);
		
		associatedLock.releaseReadLock();
		
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The READ lock of the object has been released for " + owner + ".",1);
		
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The process of releasing the READ lock for " + owner + " has ended.",2);
	}
	
	public void Lockable.releaseWriteLock(String owner)
	{
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The process of releasing the WRITE lock for " + owner + " has started.",2);
		
		Lock associatedLock = lockCollection.get(owner);
		
		associatedLock.releaseWriteLock();
		
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The WRITE lock of the object has been released for " + owner + ".",1);
		
		Debugger.printMessage("Lockable",Debugger.OBJECT_ASPECT,this,"The process of releasing the WRITE lock for " + owner + " has ended.",2);
	}
	
	public void Lockable.upgradeLock(String owner)
	{
		// Atomic (ideally)
		this.releaseReadLock(owner);
		
		this.acquireWriteLock(owner);
		// Atomic
		
	}
	public Map<String,Lock> Lockable.getLockCollection()
	{
		return lockCollection;
	}
	
	public void Lockable.setLockCollection(Map<String,Lock> newCollection)
	{
		lockCollection = newCollection;
	}
}
