package library.aspects.object;

import library.interfaces.Shared;
import library.interfaces.AccessClassified;
import library.interfaces.Lockable;
import library.util.AccessTypes;

import library.util.Debugger;

public aspect SharedAspect introduces Shared 
{
	//declare parents : Shared implements AccessClassified/*, Lockable*/;
	
	private boolean Shared.currentlyInUse = false;

	public boolean Shared.isCurrentlyInUse()
	{
		return currentlyInUse;
	}

	public void Shared.setCurrentlyInUse(boolean value)
	{
		currentlyInUse = value;
	}

	pointcut anyMethodExecution(Shared sharedObject): 
		target(sharedObject) && execution(public * Shared+.*(..)) 
			&& !execution(public * Shared.*(..));
			
	Object around(Shared sharedObject) : anyMethodExecution(sharedObject)
	{
		Debugger.printMessage("Shared",Debugger.OBJECT_ASPECT,sharedObject,"The Method Call to the object has been intercepted.",2);
		
		String accessType = sharedObject.getAccessTypeOfMethod(thisJoinPoint.getSignature().getName());
		
		if(accessType == AccessTypes.READ)
		{
			sharedObject.acquireReadLock("Shared");
			
			Debugger.printMessage("Shared",Debugger.OBJECT_ASPECT,sharedObject,"The shared READ lock of object has been acquired.",1);
		}
		else if(accessType == AccessTypes.WRITE || accessType == AccessTypes.UPDATE)
		{
			sharedObject.acquireWriteLock("Shared");
			
			Debugger.printMessage("Shared",Debugger.OBJECT_ASPECT,sharedObject,"The shared WRITE lock of object has been acquired.",1);
		}
		
		sharedObject.setCurrentlyInUse(true);		

		Debugger.printMessage("Shared",Debugger.OBJECT_ASPECT,sharedObject,"The aspect lets the call proceed.",2);
		
		Object returningObject = proceed(sharedObject);
		
		Debugger.printMessage("Shared",Debugger.OBJECT_ASPECT,sharedObject,"The aspect re-intercepts the call.",2);
		
		//On the way back, releasing stage
		if(accessType == AccessTypes.READ)
		{
			sharedObject.releaseReadLock("Shared");
			
			Debugger.printMessage("Shared",Debugger.OBJECT_ASPECT,sharedObject,"The shared READ lock of object has been released.",1);
		}
		else if(accessType == AccessTypes.WRITE || accessType == AccessTypes.UPDATE)
		{
			sharedObject.releaseWriteLock("Shared");
			
			Debugger.printMessage("Shared",Debugger.OBJECT_ASPECT,sharedObject,"The shared WRITE lock of object has been released.",1);
		}
		
		sharedObject.setCurrentlyInUse(false);
		
		Debugger.printMessage("Shared",Debugger.OBJECT_ASPECT,sharedObject,"The Method Call to the object has been returned.",2);
		
		return returningObject;
	}
}
