package library.aspects.object;

import library.interfaces.Versioned;
import library.interfaces.ContextAware;
import library.interfaces.VersionedContextAware;
import library.interfaces.ContextParticipant;
import library.interfaces.Context;

import library.util.Debugger;

public aspect VersionedContextAwareAspect introduces VersionedContextAware
{
	declare parents: VersionedContextAware implements /*Versioned, */ContextAware;

	public Integer VersionedContextAware.lookUpVersionForContext(Context context)
	{
		return getAllMappings().get(context);
	}
	
	public void VersionedContextAware.registerContextForVersion(Context context, int versionID)
	{
		getAllMappings().put(context,versionID);
		
		Debugger.printMessage("VersionedContextAware",Debugger.OBJECT_ASPECT,this,"The context [" +  context.getName() + "]" +
				" has been registered to version: -" + versionID + "-.",1);
	}
	
	public void VersionedContextAware.unregisterContext(Context context)
	{
		int versionID = getAllMappings().remove(context);
		
		Debugger.printMessage("VersionedContextAware",Debugger.OBJECT_ASPECT,this,"The context [" +  context.getName() + "]" +
				" has been unregistered from version: -" + versionID + "-.",1);
	}
	
	pointcut memberAccessesContextAwareObject(VersionedContextAware object):
    	target(object) && call(public int findCorrectVersionToInvoke(..));
	
	Object around(VersionedContextAware object) : 
										memberAccessesContextAwareObject(object)
	{	
		Debugger.printMessage("VersionedContextAware",Debugger.OBJECT_ASPECT,object,"Aspect has intercepted the call to see" +
				" whether the context has a specific version.",2);
		
		if(Thread.currentThread() instanceof ContextParticipant)
		{
			ContextParticipant participant = (ContextParticipant) Thread.currentThread();
		
			Context context = participant.getCurrentContext();
		
			if(object.lookUpVersionForContext(context) != null)
			{
				int versionID = object.lookUpVersionForContext(context);
				
				Debugger.printMessage("VersionedContextAware",Debugger.OBJECT_ASPECT,object,"The invocation target has been switched to -" +
						versionID + "-." ,1);
				
				Debugger.printMessage("VersionedContextAware",Debugger.OBJECT_ASPECT,object,"The call is returned.",2);
				
				return versionID;
			}
			else
			{
				Debugger.printMessage("VersionedContextAware",Debugger.OBJECT_ASPECT,object,"The context is not registered to any specific version.",2);
			}
		}
		
		else
		{
			Debugger.printMessage("VersionedContextAware",Debugger.OBJECT_ASPECT,object,"The call is not coming from a participant, therefore no action taken.",2);
		}
		
		
		Debugger.printMessage("VersionedContextAware",Debugger.OBJECT_ASPECT,object,"The call is returned.",2);
		
		return proceed(object);	
		
	} 
	
}
