package library.aspects.participant;

import library.context.ContextManager;
import library.interfaces.Collaborating;
import library.interfaces.Collaborative;
import library.interfaces.EntrySynchronizing;
import library.util.Debugger;
import library.util.Parameters;

public aspect CollaboratingAspect introduces Collaborating
{
	//declare parents: Collaborating implements ContextParticipant;
	
	//Interface Methods //
	public void Collaborating.createOrJoinContext(String name, String path)
	{
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant is willing to create a context named -" + name + "-.",1);
		
		Collaborative context = (Collaborative) ContextManager.createOrRetrieveContext(name, path);
		
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant is done creating a context named -" + name + "-.",2);
		
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant will join the context named -" + name + "-.",1);
		
		context.assignParticipant(this);
		
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant has joined the context named -" + name + "-.",2);
	}

	public void Collaborating.createOrJoinContext(String name, String path, int parameterCode, int parameterValue)
	{
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant is willing to create a context named -" + name + "-.",1);
		
		Collaborative context = (Collaborative) ContextManager.createOrRetrieveContext(name, path);
				
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant is done creating a context named -" + name + "-.",2);
		
		if(parameterCode == Parameters.MINIMUM_MEMBER_COUNT)
		{
			EntrySynchronizing syncContext = (EntrySynchronizing) context;
			
			if(!syncContext.getEntrySynchronizationActive())
			{
				Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant will set the minimum number of participants: " + parameterValue + "-.",1);
				
				syncContext.setMinimumMemberCount(parameterValue);
				
				Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant has set the minimum number of participants: " + parameterValue + "-.",2);
			}
		}
		else if(parameterCode == Parameters.MAXIMUM_MEMBER_COUNT)
		{
			Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant will set the maximum number of participants: " + parameterValue + "-.",1);
			
			context.setMaximumMemberCount(parameterValue);
			
			Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant has set the maximum number of participants: " + parameterValue + "-.",2);
		}
		else
		{
			System.out.println("{CollaborativeAspect} No such parameter!");
		}
		
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant will join the context named -" + name + "-.",1);
		
		context.assignParticipant(this);
		
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,this,"The participant has joined the context named -" + name + "-.",2);
	}

	public void Collaborating.closeContext()
	{
		Collaborative myContext = (Collaborative) getCurrentContext();
		
		myContext.closeContext();
	}
	//	 Interface Methods //

	
	// Make sure that spawned members become a part of context right away
	pointcut constructorExecution(Collaborating joinedMember):
		this(joinedMember) && call(public Collaborating+.new(..)) ;
	
	Object around(Collaborating joinedMember) : constructorExecution(joinedMember)
	{
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,joinedMember,"The creation call of the participant -" + Thread.currentThread().getName()
				+ "- has been intercepted.",2);
		
		//System.out.println("Collaborating aspect intercepted, and proceeds the call now.");
		Collaborating spawnedMember = (Collaborating) proceed(joinedMember);
		
		//System.out.println("Collaborating aspect regains the possesion!");
		if(joinedMember.getCurrentContext() != null)
		{
			Collaborative context = (Collaborative) joinedMember.getCurrentContext();
			context.addSpawnedParticipant(spawnedMember);
		}
		
		Debugger.printMessage("Collaborating",Debugger.PARTICIPANT_ASPECT,joinedMember,"The creation call of the participant -" + Thread.currentThread().getName()
				+ "- has been released.",2);
		
		return spawnedMember;
	}
	// Make sure that spawned members become a part of context right away
}
