package library.aspects.participant;

import library.exceptions.UnexpectedOutcomeException;
import library.interfaces.OutcomeAffecting;
import library.interfaces.OutcomeAware;
import library.interfaces.Context;
import library.interfaces.ContextParticipant;

import library.util.Debugger;

public aspect OutcomeAffectingAspect introduces OutcomeAffecting
{
	//declare parents: OutcomeAffecting implements ContextParticipant;

	private boolean OutcomeAffecting.defaultLeaveNegative = true;
	
	private boolean OutcomeAffecting.votedForContext = false;
	private boolean OutcomeAffecting.votedPositive = false;
	
	public void OutcomeAffecting.setDefaultLeaveNegative(boolean value)
	{
		defaultLeaveNegative = value;
	}

	public boolean OutcomeAffecting.getDefaultLeaveNegative()
	{
		return defaultLeaveNegative;
	}
	
	public boolean OutcomeAffecting.hasVotedForContext()
	{
		return votedForContext;
	}
	
	public void OutcomeAffecting.setVotedForContext(boolean value)
	{
		votedForContext = true;
	}
	
	public boolean OutcomeAffecting.didVotePositive()
	{
		return votedPositive;
	}
	
	public void OutcomeAffecting.votedPositive(boolean value)
	{
		votedPositive = value;
	}
	
	public void OutcomeAffecting.getNotifiedByContext(String message)
	{
		throw new UnexpectedOutcomeException(message);
	}
	
	// Interface methods
	public void OutcomeAffecting.leavePositive()
	{
		Debugger.printMessage("OutcomeAffecting",Debugger.PARTICIPANT_ASPECT,this,"The participant is willing to vote positive",1);
		
		Context currentContext = getCurrentContext();
		
		if(currentContext instanceof OutcomeAware)
		{
			OutcomeAware outcomeAwareContext = (OutcomeAware) currentContext;
			
			outcomeAwareContext.setOutcome(true);
			
			Debugger.printMessage("OutcomeAffecting",Debugger.PARTICIPANT_ASPECT,this,"The participant has voted positive.\n",2);
			
			votedForContext = true;
			votedPositive = true;
		}
		
		leaveContext();
	}
	
	public void OutcomeAffecting.leaveNegative()
	{
		Debugger.printMessage("OutcomeAffecting",Debugger.PARTICIPANT_ASPECT,this,"The participant is willing to vote negative and leave.",1);
		
		Context currentContext = getCurrentContext();
		
		if(currentContext instanceof OutcomeAware)
		{
			OutcomeAware outcomeAwareContext = (OutcomeAware) currentContext;
			
			outcomeAwareContext.setOutcome(false);
			
			Debugger.printMessage("OutcomeAffecting",Debugger.PARTICIPANT_ASPECT,this,"The participant has voted negative.\n",2);
			
			votedForContext = true;
			votedPositive = false;
		}
		
		leaveContext();
		
	}
	// End of Interface Methods
	
	pointcut outcomeAwareMemberDoesntVote(OutcomeAffecting member) :
		call(public * leaveContext(..)) && this(member) && !within(OutcomeAffectingAspect);
	
	Object around(OutcomeAffecting member) : outcomeAwareMemberDoesntVote(member)
	{
		Debugger.printMessage("OutcomeAffecting",Debugger.PARTICIPANT_ASPECT,member,"The leave request of the participant (who hasn't voted) has been intercepted..",2);
		
		Object returningObject = null;
		
		// Member hasn't voted.. Now it has to vote and quit..
		if(!member.hasVotedForContext())
		{
			Debugger.printMessage("OutcomeAffecting",Debugger.PARTICIPANT_ASPECT,member,"The participant is now forced to vote.",1);
			
			if(member.getDefaultLeaveNegative())
				member.leaveNegative();
			else
				member.leavePositive();
		}
		else
		{
			returningObject = proceed(member);
		}
		
		Debugger.printMessage("OutcomeAffecting",Debugger.PARTICIPANT_ASPECT,member,"The leave request of the participant (who hasn't voted) has been released..",2);
		
		return returningObject;
	}
	
	
}
