// This class will do intelligent System.out.println() calls.

package library.util;

import library.interfaces.*;

public class Debugger 
{
	public static final int OBJECT_ASPECT = 0;
	public static final int CONTEXT_ASPECT = 1;
	public static final int PARTICIPANT_ASPECT = 2;
	
	// Level 0: No logging at all
	// Level 1: Only log the active actions of the aspects
	// Level 2: Log each interception, each action with every detail
	public static final int VERBOSITY = 2;
	
	public static void printMessage(String aspectName, int aspectType, Object object, String message, int level)
	{
		String objectName = determineName(aspectType,object);
		
		if(VERBOSITY >= level)
		{	
			System.out.println(prepareFinalMessage(aspectName,objectName,message));
		}
	}
	
	public static StringBuffer prepareFinalMessage(String aspectName, String objectName, String message)
	{
		StringBuffer result = new StringBuffer("{" + aspectName + "}");
		
		for(int i = result.length(); i < 33 ; i++)
			result.append(" ");
		
		result.append("<" + objectName + ">");
		
		for(int i = result.length(); i < 58 ; i++)
			result.append(" ");
		
		result.append(" -- " + message);
		
		return result;
		
	}
	
	private static String determineName(int type, Object object)
	{
		if(type == OBJECT_ASPECT)
		{
			if(object instanceof Named)
			{
				Named named = (Named) object;
				return named.getMyName();
			}
			else
			{
				return "unknown";
			}
		}
		
		else if(type == CONTEXT_ASPECT)
		{
			Context context = (Context) object;
			return context.getName();
		}
		
		else
		{
			Thread thread = (Thread) object;
			return thread.getName();
		}
	}
}
