package library.util;

import library.util.Semaphore;
import java.io.Serializable;

public class Lock implements Serializable
{
	
	transient private int numberOfReaders; // number of readers
	
	private Semaphore readerSemaphore; // synchronizing access to nr
	private Semaphore readWriteSemaphore; // synchronizing Read/Write access to an object
	
	transient private String readLockHolder, writeLockHolder, updateLockHolder; // holders of the respective locks
	transient private String method; // the name of the method on which a lock was obtained

	public Lock() 
	{
		numberOfReaders = 0;
		
		readerSemaphore = new Semaphore();
		readWriteSemaphore = new Semaphore(); 
		
		readLockHolder = "#";
		writeLockHolder = "#";
		updateLockHolder = "#";
		method = "#";
		
	}
	 
	
	/**
	 * Get read lock. 
	 * 
	 */
	public void getReadLock(String holder, String name)
	{
		readerSemaphore.P();
		numberOfReaders = numberOfReaders + 1;
		
		if(numberOfReaders == 1)
		{
			readWriteSemaphore.P();
			//System.out.println("------------------Someone got the read lock!-----------");
		}
		readLockHolder = holder;
		method = name;
		
		
		readerSemaphore.V();

	}

	public void releaseReadLock()
	{
		readerSemaphore.P();
		numberOfReaders = numberOfReaders - 1;

		if(numberOfReaders == 0)
		{
			readLockHolder = "";
			method = "";
			readWriteSemaphore.V();
			//System.out.println("------------------Someone released the read lock!-----------");
		}				
	
	    readerSemaphore.V();
 
	
	}
	
	public void getWriteLock(String holder, String name)
	{
		readWriteSemaphore.P();
		//System.out.println("------------------Someone got the write lock!-----------");
		writeLockHolder = holder;
		method = name;
	}
	

	public void releaseWriteLock()
	{
		writeLockHolder = "";
		method = "";
			  	 
		readWriteSemaphore.V();
		//System.out.println("------------------Someone released the write lock!-----------");
				
	}
	
	public void getUpdateLock(String holder,String name)
	{
		readWriteSemaphore.P();
		updateLockHolder = holder;
		method = name;
	}
	
	public void releaseUpdateLock()
	{
		updateLockHolder = "";
		method = "";
			  
		readWriteSemaphore.V();

	}
	
	public String getReadLockHolder()
	{
		return readLockHolder;
	}
	
	public String getWriteLockHolder()
	{
		return writeLockHolder ;
	}
	
	public String getUpdateLockHolder()
	{
		return updateLockHolder;
	}
	
	public String getMethodName()
	{
		return method;
	}
	
}
